#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
"""
Demonstration on how the algorithm of an MX function can be accessed and its operations can be transversed.
"""

from casadi import *
import numpy

# Create a function
a = MX.sym('a')
b = MX.sym('b',2)
c = MX.sym('c',2,2)
f = Function("f", [a,b,c], [3*mtimes(c,b)*a + b], ['a', 'b', 'c'], ['r'])

# Input values of the same dimensions as the above
input_val = [numpy.array([2.0]),\
             numpy.array([3.0,4.0]),\
             numpy.array([[5.0,1.0],[8.0,4.0]])]

# Output values to be calculated of the same dimensions as the above
output_val = [numpy.zeros(2)]

# Work vector
work = [ None for i in range(f.sz_w())]

# Loop over the algorithm
for k in range(f.n_instructions()):

  # Get the atomic operation
  op = f.instruction_id(k)

  o = f.instruction_output(k)
  i = f.instruction_input(k)

  if(op==OP_CONST):
    v = f.instruction_MX(k).to_DM()
    assert v.is_dense()
    work[o[0]] = np.array(v)
    print('work[{o[0]}] = {v}'.format(o=o,v=v))
  else:
    if op==OP_INPUT:
      work[o[0]] = input_val[i[0]]
      print('work[{o[0]}] = input[{i[0]}]            ---> {v}'.format(o=o,i=i,v=work[o[0]]))
    elif op==OP_OUTPUT:
      output_val[o[0]] = work[i[0]]
      print('output[{o[0]}] = work[{i[0]}]             ---> {v}'.format(o=o,i=i,v=output_val[o[0]]))
    elif op==OP_ADD:
      work[o[0]] = work[i[0]] + work[i[1]]
      print('work[{o[0]}] = work[{i[0]}] + work[{i[1]}]      ---> {v}'.format(o=o,i=i,v=work[o[0]]))
    elif op==OP_MUL:
      work[o[0]] = work[i[0]] * work[i[1]]
      print('work[{o[0]}] = work[{i[0]}] * work[{i[1]}]        ---> {v}'.format(o=o,i=i,v=work[o[0]]))
    elif op==OP_MTIMES:
      work[o[0]] = np.dot(work[i[1]], work[i[2]])+work[i[0]]
      print('work[{o[0]}] = work[{i[1]}] @ work[{i[2]}] + work[{i[0]}]        ---> {v}'.format(o=o,i=i,v=work[o[0]]))
    else:
      disp_in = ["work[" + str(a) + "]" for a in i]
      debug_str = print_operator(f.instruction_MX(k),disp_in)
      raise Exception('Unknown operation: ' + str(op) + ' -- ' + debug_str)
      # When evaluating with MX, you may use an identity transformation:
      #node = f.instruction_MX(k)
      
      #args = [work[ik] for ik in i]
      #res = node.eval_mx(args)
      #for ok,e in zip(o,res):
      #  work[ok] = e

print('------')
print('Evaluated ' + str(f))
print('Expected: ', f.call(input_val))
print('Got:      ', output_val)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from __future__ import print_function
import os
import sys
from subprocess import *
import time
import re
import tempfile

class TimeoutEvent(Exception):
    pass

def alarm_handler(signum, frame):
    raise TimeoutEvent


def is_exe(root,name):
  fpath = os.path.join(root,name)
  return os.path.exists(fpath) and os.access(fpath, os.X_OK)

from os import kill
from signal import signal


try:
    from signal import alarm, SIGALRM, SIGKILL
    alarm_available = True
except:
    def alarm(amount):
        pass
    alarm_available = False

from subprocess import PIPE, Popen

# Snippet from http://stackoverflow.com/questions/1191374/subprocess-with-timeout
def run(args, input=None, cwd = None, shell = False, kill_tree = True, timeout = -1, env = None):
    '''
    Run a command with a timeout after which it will be forcibly
    killed.
    '''
    class Alarm(Exception):
        pass
    def alarm_handler(signum, frame):
        raise Alarm
    my_env = os.environ.copy()
    if env is not None:
        my_env.update(env)
    p = Popen(args, shell = shell, cwd = cwd, stdout = PIPE, stderr = PIPE, env = my_env)
    if timeout != -1:
        if alarm_available:
            signal(SIGALRM, alarm_handler)
        alarm(timeout)
    try:
        t0 = time.time()
        stdout, stderr = p.communicate()
        print("Ran for", time.time()-t0, "seconds.")
        if timeout != -1:
            alarm(0)
    except Alarm:
        pids = [p.pid]
        if kill_tree:
            pids.extend(get_process_children(p.pid))
        for pid in pids:
            # process might have died before getting to this line
            # so wrap to avoid OSError: no such process
            try:
                kill(pid, SIGKILL)
            except OSError:
                pass
        return -9, '', ''
    return p.returncode, stdout, stderr

def killProcess(pid):
  pids = [pid]
  if kill_tree:
      pids.extend(get_process_children(p.pid))
  for pid in pids:
      # process might have died before getting to this line
      # so wrap to avoid OSError: no such process
      try:
          kill(pid, SIGKILL)
      except OSError:
          pass

def get_process_children(pid):
    p = Popen('ps --no-headers -o pid --ppid %d' % pid, shell = True,
              stdout = PIPE, stderr = PIPE)
    stdout, stderr = p.communicate()
    return [int(p) for p in stdout.split()]

if __name__ == '__main__':
    print(run('find /', shell = True, timeout = 3))
    print(run('find', shell = True))

deprecated = re.compile(r"\b[dD]epr[ei]c[ie]?at[ei]")
warning = re.compile("warning")

class TestSuite:
  def __init__(self,suffix=None,dirname=None,preRun=None,postRun=None,command=None,skipdirs=["hock_schittkowski"],skipfiles=[],inputs={},workingdir = lambda x: x,allowable_returncodes=[],args=[],stderr_trigger=[],stdout_trigger=[],stdout_must_have=[],check_depreciation=True,check_warning=False,custom_stdout=None,default_fail=False):
    """

    dirname: The directory that should be crawled for test problems.

    command: A mapping of (dirname,filename) -> Popen command list.
             e.g  lambda dir,fn:  ['./'+fn]  for executable files

    skipdirs: A list of directories that should be skipped during recursive traversal of dirname.

    skipfiles: A list of matched files that should be skipped during recursive traversal of dirname.

    suffix:  Only the files that end with this suffix will be tested.
             Omit argument or set to None to test any file.

    inputs:  Can be a dictionary of filename -> string to pass numbers to STDIN of the process

    workingdir: Specify the working directory for the process. The path may be relative to the /trunk/test path


    stderr_trigger: list of strings/regexes. If any string is found in std_err, it is considered an error, regardless of the return_code. A message may be attached to a trigger by packing the string/regex in a 2-tuple, with the second element the message.

    stdout_trigger: list of strings/regexes. If any string is found in std_out, it is considered an error, regardless of the return_code. A message may be attached to a trigger by packing the string/regex in a 2-tuple, with the second element the message.

    stdout_must_have: list of strings/regexes. If any string is not found in std_out, it is considered an error, regardless of the return_code. A message may be attached to a trigger by packing the string/regex in a 2-tuple, with the second element the message.

    check_depreciation: raise an error if the output contains a depreciation warning

    check_warning: raise an error if the output contains a warning

    args: a list of command line options:
       -skipfiles="file1 file2"   get's added to skipfiles
       -memcheck           Include a check for memory leaks
       -passoptions="option1 option2"   get's passed onto the command constructor as third argument


    """
    if alarm_available:
        signal(SIGALRM, alarm_handler)

    # Don't buffer
    if sys.version_info < (3, 0):
      sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
      sys.stderr = os.fdopen(sys.stderr.fileno(), 'w', 0)

    self.stats={'numtests':0,'numfails':0}
    self.suffix = suffix
    self.dirname = dirname
    self.preRun = preRun
    self.postRun = postRun
    self.command = command
    self.skipdirs = skipdirs
    self.skipfiles = skipfiles
    self.inputs = inputs
    self.allowable_returncodes = allowable_returncodes
    self.workingdir = workingdir
    self.memcheck = False
    self.passoptions = []
    self.stderr_trigger = stderr_trigger
    self.stdout_trigger = stdout_trigger
    self.stdout_must_have = stdout_must_have
    self.custom_stdout = custom_stdout
    self.default_fail=default_fail
    if check_depreciation:
      self.stderr_trigger.append((deprecated,"deprecated"))
      self.stdout_trigger.append((deprecated,"deprecated"))
    if check_warning:
      self.stderr_trigger.append((warning,"warning"))
      self.stdout_trigger.append((warning,"warning"))
    self.args=args
    for arg in args:
      okay = False
      m = re.search('-skipfiles=(.*)', arg)
      if m:
        self.skipfiles+=m.group(1).split(' ')
        okay = True
      m = re.search('-skipdirs=(.*)', arg)
      if m:
        self.skipdirs+=m.group(1).split(' ')
        okay = True
      m = re.search('-memcheck', arg)
      if m:
        self.memcheck = True
        okay = True
      m = re.search('-passoptions=(.*)', arg)
      if m:
        self.passoptions+=m.group(1).split(' ')
        okay = True
      if not(okay):
        print("Unknown argument: ", arg)

  def run(self):
    print("Running test in " + self.dirname)
    for root, dirs, files in os.walk(self.dirname):
      if not(self.preRun is None):
        self.preRun(root)
      for name in files:
        for skip in self.skipdirs:
          if skip in dirs:
            dirs.remove(skip)
        if (self.suffix is None and is_exe(root,name)) or (not(self.suffix is None) and name.endswith('.'+self.suffix)):
          if not(name in self.skipfiles):
            self.test(root,name,self.command)
      if not(self.postRun is None):
        self.postRun(root)

    print("Ran %d tests, %d fails." % (self.stats['numtests'],self.stats['numfails']))

    if self.stats['numfails']>0:
      sys.exit(1)


  def test(self,dir,fn,commands):
    self.stats['numtests']+=1
    print(("%02d. " % self.stats['numtests']) + fn)
    t0 = time.time()

    cmd = self.command(dir,fn,self.passoptions)
    print(cmd)
    p=Popen(cmd,cwd=self.workingdir(dir),stdout=PIPE, stderr=PIPE, stdin=PIPE)

    inp = None
    if callable(self.inputs):
      inputs = self.inputs(dir,fn)
    else:
      inputs = self.inputs
    if fn in inputs:
      inp = inputs[fn]

    if inp is not None:
      inp = inp.encode("ascii")

    alarm(15*60) # 15 mins
    try:
      stdoutdata, stderrdata = p.communicate(inp)
    except TimeoutEvent:
      killProcess(p.pid)
      raise Exception("Timout.")
    try:
      stdoutdata = stdoutdata.decode("ascii",errors='ignore')
    except Exception as e:
      print(e)
    try:
      stderrdata = stderrdata.decode("ascii",errors='ignore')
    except Exception as e:
      print(e)
            
            
    alarm(0) # Remove alarm
    t = time.time() - t0
    if self.custom_stdout is not None:
      stdoutdata = self.custom_stdout(dir,fn)
    
    print("Ran for",t, "seconds")

    stderr_trigger = False
    for trigger in self.stderr_trigger:
      trigger_message = str(trigger)
      if isinstance(trigger,tuple):
        trigger_message = trigger[1]
        trigger = trigger[0]
      trigger_raised = re.search(trigger, stderrdata)
      if trigger_raised:
        print("stderr_trigger '%s' was raised." % trigger_message)
        stderr_trigger = True
    stdout_trigger = False
    for trigger in self.stdout_trigger:
      trigger_message = str(trigger)
      if isinstance(trigger,tuple):
        trigger_message = trigger[1]
        trigger = trigger[0]
      trigger_raised = re.search(trigger, stdoutdata)
      if trigger_raised:
        print("stdout_trigger '%s' was raised." % trigger_message)
        stdout_trigger = True
        
    if self.default_fail:
      passed = stdout_trigger or stderr_trigger
    else:
      passed = (not(stderr_trigger) and (p.returncode==0 or (p.returncode in self.allowable_returncodes)))
    
    stdout_must_haves = True
    for trigger in self.stdout_must_have:
      trigger_message = str(trigger)
      if isinstance(trigger,tuple):
        trigger_message = trigger[1]
        trigger = trigger[0]
      trigger_raised = re.search(trigger, stdoutdata)
      if not trigger_raised:
        print("stdout_trigger '%s' was raised." % trigger_message)
        stdout_must_haves = False

    passed = passed and stdout_must_haves
        
    if passed:
      pass
      #print "  > Succes: %0.2f [ms]" % (t*1000)
    else :
      self.stats['numfails']+=1
      print("In %s, %s failed:" % (dir,fn))
      print("="*30)
      print("returncode: ", p.returncode)
      print(stdoutdata)
      print(stderrdata)
      print("="*30)
      return

    if self.memcheck:
      # --suppressions=../internal/valgrind-python.supp
      suppressions = ["internal/valgrind-python.supp","internal/valgrind-casadi.supp"]
      supps = ['--suppressions='+os.path.join(os.getcwd(),s) for s in suppressions]
      stdoutfile = tempfile.TemporaryFile()
      p=Popen(['valgrind','--leak-check=full']+supps+['--show-possibly-lost=no','--error-limit=no',"--gen-suppressions=all"]+self.command(dir,fn,self.passoptions),cwd=self.workingdir(dir),stdout=stdoutfile, stderr=PIPE, stdin=PIPE)
      f=Popen(['grep','-E','-C','50', "definitely lost|leaks|ERROR SUMMARY|Invalid write|casadi"],stdin=p.stderr,stdout=PIPE)
      p.stderr.close()
      t0 = time.time()
      alarm(60*60) # 1 hour
      try:
        stdoutdata, stderrdata = f.communicate()
        try:
          stdoutdata = stdoutdata.decode("ascii",errors='ignore')
        except Exception as e:
          print(e)
        try:
          stderrdata = stderrdata.decode("ascii",errors='ignore')
        except Exception as e:
          print(e)
        s = stdoutfile.read()
        try:
            s = s.decode("ascii",errors='ignore')
        except:
            pass
        stdoutdata = s + "\n"+ stdoutdata
      except TimeoutEvent:
        killProcess(p.pid)
        killProcess(f.pid)
        raise Exception("Timeout.")
      alarm(0) # Remove alarm
      print("Ran for", time.time()-t0, "seconds")
      m = re.search('definitely lost: (.*) bytes', stdoutdata)
      lost = "0"
      if m:
        lost = m.group(1)
      else:
        m = re.search("no leaks are possible",stdoutdata)
        if not(m):
          if p.returncode==0:
            return
          debug = str(stdoutdata) + " -- " + str(stderrdata) + " -- " + str(f.returncode)
          raise Exception("valgrind output is not like expected: %s" % debug)

      m = re.search('ERROR SUMMARY: (.*) errors', stdoutdata)
      errors = "0"
      if m:
        errors = m.group(1)
      else:
        print(stdoutdata)
        raise Exception("valgrind output is not like expected: %s")

      error_casadi = False
      # Filter out stuff after address header
      diagnose_lines = []
      error_log = True
      for l in stdoutdata.split("\n"):
        if l.startswith("=="):
          if "  Address" in l:
            error_log = False
          elif "  at" in l or "  by" in l:
            pass
          else:
            error_log = True

        if error_log:
          diagnose_lines.append(l)
          if l.startswith("==") and re.search('casadi', l):
            error_casadi = True

      diagnosis = "\n".join(diagnose_lines)

      errors = "0"  # disabling valgrind error-checking for now: samples are flooded with errors
      if not(lost=="0" and errors=="0") or error_casadi:
        if not(lost=="0"):
          print("Memory leak: lost %s bytes" % (lost))
        if not(errors=="0"):
          print("Valgrind errors: %s" % (errors))
        print("="*30)
        print(stdoutdata)
        print("="*30)
        self.stats['numfails']+=1




#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2014 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             K.U. Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
import zipfile
import os
import casadi
release = casadi.__version__

import zlib
compression = zipfile.ZIP_DEFLATED
zf = zipfile.ZipFile('example_pack.zip', mode='w')


if "+" in release:
  releasedocurl = "http://docs.casadi.org/api/html/annotated.html"
else:
  releasedocurl = "http://docs.casadi.org/v%s/api/html/annotated.html" % release

zf.writestr('README.txt',"""

This is a collection of python examples for CasADi %s.
Consult the documentation online at %s.

To run an example vdp_single_shooting.py, you have several options:

a) fire up a terminal, navigate to this directory, and type "python vdp_single_shooting.py"
b) fire up a terminal, navigate to this directory, and type "ipython" and then "run vdp_single_shooting.py"
c) start spyder, open vdp_single_shooting.py and press "run".

    
""" % (release,releasedocurl))

base = "../examples/python"
for root, dirs, files in os.walk(base): # Walk directory tree
  for f in files:
    if f.endswith(".py"):
       zf.write(os.path.join(root,f),os.path.join("python",f),compress_type=compression)
base = "../examples/matlab"
for root, dirs, files in os.walk(base): # Walk directory tree
  for f in files:
    if f.endswith(".m"):
       zf.write(os.path.join(root,f),os.path.join("matlab",f),compress_type=compression)
       
base = "../documents"
for root, dirs, files in os.walk(base): # Walk directory tree
  for f in files:
    if f.endswith(".pdf"):
       zf.write(os.path.join(root,f),f,compress_type=compression)

#zf.write("../cheatsheet/python.pdf","cheatsheet.pdf",compress_type=compression)
zf.write("../users_guide/users_guide.pdf","user_guide.pdf",compress_type=compression)

zf.close()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import numpy

from helpers import *

class Symbolictests(casadiTestCase):

  @slow()
  @memory_heavy()
  def test_in_place_simplification(self):
    print("In place simplification")

    variables = ["a","b"]
    numbers = [1.3,0.7001]

    binary_operations = [lambda x,y :"(%s+%s)" % (x,y),lambda x,y: "(%s-%s)" % (x,y),lambda x,y :"(%s/%s)" % (x,y),lambda x,y: "(%s*%s)" % (x,y)]
    unary_operations = [lambda x: "(-%s)" % x, lambda x: x]


    """

       b1          b2               b3       b4
             op1                        op2
                           op3
    """


    def nodes():
      for b in variables:
        for u in unary_operations:
          yield u(b)

    def operations(a,b):
      for op in binary_operations:
        for u in unary_operations:
          yield u(op(a,b))
      for u in unary_operations:
        yield u(a)
        yield u(b)

    def operations_toplevel(a,b):
      for op in binary_operations:
        yield op(a,b)

    def operations_node():
      for b1 in nodes():
        for b2 in nodes():
          for op in operations(b1,b2):
            yield op
      for b1 in nodes():
        yield b1

    for a_s, b_s in [(SX.sym("a"),SX.sym("b")),(MX.sym("a"),MX.sym("b"))]:
      i=0
      for op1 in operations_node():
        for op2 in operations_node():
           for op3 in operations_toplevel(op1,op2):
             i+= 1
             e = eval(op3,{"a": a_s, "b": b_s})
             f = Function("f", [a_s,b_s],[e])
             #print i, op1, op2, op3, " -> ", e
             r = f(numbers[0], numbers[1])
             try:
               num = eval(op3,{"a": numbers[0], "b":  numbers[1]})
             except ZeroDivisionError:
               num = None
             if num is None:
               self.assertTrue(numpy.isnan(float(r)) or numpy.isinf(float(r)))
               continue
             if (abs(num-r)>1e-10):
               print(i , op3, " -> ", e)
               self.assertTrue(False)

if __name__ == '__main__':
    unittest.main()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *

# Example how to export FMUs from CasADi
# Joel Andersson, joel@jaeandersson.com

# Start with an empty DaeBuilder instance
dae = DaeBuilder('vdp')

# States
t = dae.add('t', 'independent')
x1 = dae.add('x1', 'output', dict(start = 1))
x2 = dae.add('x2', 'output', dict(start = 0))
q = dae.add('q')

# Input
u = dae.add('u', 'input')

# Set ODE right-hand-sides
dae.eq(dae.der(x1), (1 - x2 * x2)*x1 - x2 + u)
dae.eq(dae.der(x2), x1)
dae.eq(dae.der(q), x1**2 + x2**2 + u**2)

# Add bounds
dae.set_min('u', -0.75)
dae.set_max('u', 1)
dae.set_start('u', 0.5)

# Print DAE
dae.disp(True)

# Export FMU
funcs = dae.export_fmu()
print('Generated files: {}'.format(funcs))

# Compile DLL
import os
casadi_root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
os.system(f'gcc --shared -fPIC -I{casadi_root}/external_packages/FMI-Standard-3.0/headers/ vdp.c vdp_wrap.c -o vdp.so')
print('Compiled vdp.so')

# Package into an FMU
import zipfile
fmuname = 'vdp_generated.fmu'
with zipfile.ZipFile(fmuname, 'w') as fmufile:
    # Add generated files to the archive
    for f in funcs:
      arcname = f if f == 'modelDescription.xml' else 'sources/' + f
      fmufile.write(f, arcname = arcname)
      os.remove(f)
    # Add compile DLL to the archive (assume Linux 64 bit)
    fmufile.write('vdp.so', arcname = 'binaries/x86_64-linux/vdp.so')
    os.remove('vdp.so')
print(f'Created FMU: {fmuname}')

# Load the FMU in FMPy
try:
    import fmpy
    fmpy.dump(fmuname)
    # Simulate the generated FMU
    res = fmpy.simulate_fmu(fmuname)
    import matplotlib.pyplot as plt
    plt.figure(1)
    plt.clf()
    plt.plot(res['time'], res['x1'],'-', label = 'x1')
    plt.plot(res['time'], res['x2'],'--', label = 'x2')
    plt.xlabel('time')
    plt.legend()
    plt.grid()
    plt.show()

except ImportError as e:
   print('FMPy not installed. Skipping FMU simulation.')



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import numpy as n
import unittest
from types import *
from helpers import *



scipy_available = True
try:
	import scipy.special
	from scipy.linalg import expm
except:
	scipy_available = False

class Simulatortests(casadiTestCase):


  def setUp(self):
    # Reference solution is q0 e^((t^3-t0^3)/(3 p))
    t=SX.sym('t')
    q=SX.sym('q')
    p=SX.sym('p')
    f={'t':t, 'x':q, 'p':p, 'ode':q/p*t**2}
    opts = {}
    opts['reltol'] = 1e-15
    opts['abstol'] = 1e-15
    opts['fsens_err_con'] = True
    #opts['verbose'] = True
    integrator = casadi.integrator('integrator', 'cvodes', f, 0, 2.3, opts)
    q0   = MX.sym('q0')
    par  = MX.sym('p')
    qend = integrator.call({'x0':q0,'p':par})['xf']
    qe=Function('qe', [q0,par],[qend])
    self.dae = f
    self.integrator = integrator
    self.qe=qe
    self.qend=qend
    self.q0=q0
    self.par=par
    self.f = f
    self.num={'tend':2.3,'q0':7.1,'p':2}
    pass

  def test_sim_full(self):
    self.message('Simulator inputs')
    num = self.num
    N = 4
    tc = n.linspace(0,num['tend'], N)

    t=SX.sym('t')
    q=SX.sym('q')
    p=SX.sym('p')

    f={'t':t, 'x':q, 'p':p, 'ode':q/p*t**2}
    opts = {}
    opts['reltol'] = 1e-15
    opts['abstol'] = 1e-15
    opts['fsens_err_con'] = True
    #opts['verbose'] = True
    integrator = casadi.integrator('integrator', 'cvodes', f, tc[0], tc, opts)

    solution = Function('solution', {'x0':q, 'p':p, 'xf':horzcat(*[q*exp(t**3/(3*p)) for t in tc])},
                        casadi.integrator_in(), casadi.integrator_out())
    f_in = {}
    f_in['x0']=0.3
    f_in['p']=0.7

    self.checkfunction(integrator,solution,inputs=f_in,adj=False,jacobian=False,sens_der=False,evals=False,digits=6)

  def test_simulator_time_offset(self):
    self.message('CVodes integration: simulator time offset')
    num=self.num
    t = n.linspace(0.7,num['tend'],100)

    opts = {}
    opts['reltol'] = 1e-15
    opts['abstol'] = 1e-15
    opts['fsens_err_con'] = True
    integrator = casadi.integrator('integrator', 'cvodes', self.dae, t[0], t, opts)

    x0_ind = casadi.integrator_in().index('x0')
    integrator_in = [0]*integrator.n_in();integrator_in[x0_ind]=[num['q0']]
    p_ind = casadi.integrator_in().index('p')
    integrator_in[p_ind]=[num['p']]
    integrator_out = integrator.call(integrator_in)

    tend=num['tend']
    q0=num['q0']
    p=num['p']

    self.assertAlmostEqual(integrator_out[0][0,-1],q0*exp((tend**3-0.7**3)/(3*p)),9,'Evaluation output mismatch')

if __name__ == '__main__':
    unittest.main()




#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *
import numpy as NP
import matplotlib.pyplot as plt

# Declare variables (use simple, efficient DAG)
x0=SX.sym("x0"); x1=SX.sym("x1")
x = vertcat(x0,x1)

# Control
u = SX.sym("u")

# ODE right hand side
xdot = vertcat((1 - x1*x1)*x0 - x1 + u, x0)

# Lagrangian function
L = x0*x0 + x1*x1 + u*u

# Costate
lam = SX.sym("lam",2)

# Hamiltonian function
H = dot(lam,xdot) + L

# Costate equations
ldot = -gradient(H,x)

## The control must minimize the Hamiltonian, which is:
print("Hamiltonian: ", H)

# H is of a convex quadratic form in u: H = u*u + p*u + q, let's get the coefficient p
p = gradient(H,u)     # this gives us 2*u + p
p = substitute(p,u,0) # replace u with zero: gives us p

# H's unconstrained minimizer is: u = -p/2
u_opt = -p/2

# We must constrain u to the interval [-0.75, 1.0], convexity of H ensures that the optimum is obtain at the bound when u_opt is outside the interval
u_opt = fmin(u_opt,1.0)
u_opt = fmax(u_opt,-0.75)
print("optimal control: ", u_opt)

# Augment f with lam_dot and subtitute in the value for the optimal control
f = vertcat(xdot,ldot)
f = substitute(f,u,u_opt)

# Function for obtaining the optimal control from the augmented state
u_fcn = Function("ufcn", [vertcat(x,lam)], [u_opt])

# Formulate the DAE
dae = {'x':vertcat(x,lam), 'ode':f}

# Augmented DAE state dimension
nX = 4

# End time
tf = 10.0

# Number of shooting nodes
num_nodes = 20

# Create an integrator (CVodes)
iopts = {}
iopts["abstol"] = 1e-8 # abs. tolerance
iopts["reltol"] = 1e-8 # rel. tolerance
I = integrator("I", "cvodes", dae, 0, tf/num_nodes, iopts)

# Variables for the states at each shooting node
X = MX.sym('X',nX,num_nodes+1)

# Formulate the root finding problem
G = []
G.append(X[:2,0] - vertcat(0,1)) # states fixed, costates free at initial time
for k in range(num_nodes):
  XF = I(x0=X[:,k])["xf"]
  G.append(XF-X[:,k+1])
G.append(X[2:,num_nodes] - vertcat(0,0)) # costates fixed, states free at final time

# Terminal constraints: lam = 0
rfp = {"x": vec(X), "g": vertcat(*G)}

# Select a solver for the root-finding problem
Solver = "nlpsol"
#Solver = "newton"
#Solver = "kinsol"

# Solver options
opts = {}
if Solver=="nlpsol":
    opts["nlpsol"] = "ipopt"
    opts["nlpsol_options"] = {"ipopt.hessian_approximation":"limited-memory"}
elif Solver=="newton":
    opts["linear_solver"] = "csparse"
elif Solver=="kinsol":
    opts["linear_solver_type"] = "user_defined"
    opts["linear_solver"] = "csparse"
    opts["max_iter"] = 1000

# Allocate a solver
solver = rootfinder('solver', Solver, rfp, opts)

# Solve the problem
X_sol = solver(x0=0)['x']
print(X_sol)

# Time grid for visualization
tgrid = NP.linspace(0,tf,100)

# Simulator to get optimal state and control trajectories
simulator = integrator('simulator', 'cvodes', dae, 0, tgrid)

# Simulate to get the trajectories
sol = simulator(x0 = X_sol[0:4])["xf"]

# Calculate the optimal control
u_opt = u_fcn(sol)

# Plot the results
plt.figure(1)
plt.clf()
plt.plot(tgrid, sol[0, :].T, '--')
plt.plot(tgrid, sol[1, :].T, '-')
plt.plot(tgrid, u_opt.T, '-.')
plt.title("Van der Pol optimization - indirect multiple shooting")
plt.xlabel('time')
plt.legend(['x trajectory','y trajectory','u trajectory'])
plt.grid()
plt.show()

print(sol[0,:])



# This is an automatically generated file converting from the apm format
import casadi as cs


def hs110():
    # The optimal objective is (if given in):
    f_opt = -45.77846971
    x_opt = cs.DM([9.35027, 9.35027, 9.35027, 9.35027, 9.35027, 9.35027, 9.35027, 9.35027, 9.35027, 9.35027])
    x = cs.MX.sym('x', 10)
    obj = cs.MX.zeros(10, 1)
    x0 = cs.DM.zeros(10, 1)
    lbx = -cs.inf*cs.DM.ones(10, 1)
    ubx = cs.inf*cs.DM.ones(10, 1)
    g = []
    lbg = []
    ubg = []

    x0[0:10] = 9
    lbx[0:10] = 2.001
    ubx[0:10] = 9.999

    prod = 1
    for i in range(10):
        prod = prod*x[i]**0.2

    obj[0:10] = cs.log(x[0:10]-2)**2 + cs.log(10-x[0:10])**2

    f = cs.sum1(obj) - prod

    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#


from casadi import *
import casadi as c
import numpy
import numpy as n
from numpy import array, matrix
import unittest
from types import *
from helpers import *

scipy_available = True
try:
	from scipy.sparse import *
	from scipy import *
except:
	scipy_available = False
import warnings

class typemaptests(casadiTestCase):

  def setUp(self):
    pass

  def test_floordiv(self):
    self.message("make sure that floor_div raises errors")
    for x in [SX.sym("x"),MX.sym("x"),SX.sym("x")]:
      for y in [2,2.0,DM(3),numpy.array([2.0])]:
        print((x,y))
        self.assertRaises(Exception,lambda : x//y)
        self.assertRaises(Exception,lambda : y//x)

  def test_autoconversion(self):
    self.message("Auto conversion DM")
    x=np.array([2.3])
    s = DM([[1,2],[3,4]])
    n = np.array(s)

    self.checkarray(x[0]*s,s*x[0],"")
    self.checkarray(x[0]*s,n*x[0],"")

    self.checkarray(x[0]/s,1/(s/x[0]),"")
    self.checkarray(x[0]/s,x[0]/n,"")

    self.checkarray(x[0]-s,-(s-x[0]),"")
    self.checkarray(x[0]-s,x[0]-n,"")

    self.checkarray(x[0]+s,s+x[0],"")
    self.checkarray(x[0]+s,x[0]+n,"")

    w=np.array([2.3])[0]
    w+=s
    self.checkarray(w,2.3+n,"")

    w=np.array([2.3])[0]
    w-=s
    self.checkarray(w,2.3-n,"")

    w=np.array([2.3])[0]
    w*=s
    self.checkarray(w,2.3*n,"")

    w=np.array([2.3])[0]
    w/=s
    self.checkarray(w,2.3/n,"")

    x=[2.3]

    self.checkarray(x[0]*s,s*x[0],"")
    self.checkarray(x[0]*s,n*x[0],"")

    self.checkarray(x[0]/s,1/(s/x[0]),"")
    self.checkarray(x[0]/s,x[0]/n,"")

    self.checkarray(x[0]-s,-(s-x[0]),"")
    self.checkarray(x[0]-s,x[0]-n,"")

    self.checkarray(x[0]+s,s+x[0],"")
    self.checkarray(x[0]+s,x[0]+n,"")


    w=2.3
    w+=s
    self.checkarray(w,2.3+n,"")

    w=2.3
    w-=s
    self.checkarray(w,2.3-n,"")

    w=2.3
    w*=s
    self.checkarray(w,2.3*n,"")

    w=2.3
    w/=s
    self.checkarray(w,2.3/n,"")

  def test_autoconversionMX(self):
    self.message("Auto conversion MX")
    s = DM([[1,2],[3,4]])
    x = SX(3)
    y = MX(3)

    def doit(z,s,fun):
      if type(z) in [type(SX()),type(SX())]:
        ztype = [type(SX()),type(SX())]

      if type(z) in [type(MX())]:
        ztype = [type(MX())]

      r = fun(z,s)

      if type(z) is type(SX()) and type(s) is type(SX()):
        self.assertTrue(type(r) is type(SX()))


      self.assertTrue(type(r) in ztype)

      hasNum = True
      if type(s) in [type(SX()),type(MX()),type(SX())]:
        hasNum = False

      if hasNum:
        dummy = [1.3,2.7,9.4,1.0]

        f=Function('f', [z],[r])
        f_in = DM(f.sparsity_in(0),dummy[0:f.nnz_in(0)])
        f_out = f(f_in)

        f_=Function('f', [z],[z])
        f__in = DM(f_.sparsity_in(0),dummy[0:f.nnz_in(0)])
        f__out = f_(f__in)


        self.checkarray(fun(f__out,DM(s)),f_out,"operation")
      else:
        dummy = [1.3,2.7,9.4,1.0]
        dummy2 = [0.3,2.4,1.4,1.7]

        f=Function('f',[z,s],[r])
        f_in = [DM(f.sparsity_in(0),dummy[0:f.nnz_in(0)]), DM(f.sparsity_in(1),dummy2[0:f.nnz_in(1)])]
        f_out = f(*f_in)

        f_=Function('f', [z,s],[z,s])
        f__in= [DM(f_.sparsity_in(0),dummy[0:f.nnz_in(0)]), DM(f_.sparsity_in(1),dummy2[0:f.nnz_in(1)])]
        f__out = f_(*f__in)

        self.checkarray(fun(f__out[0],f__out[1]),f_out,"operation")


    def tests(z,s):
      doit(z,s,lambda z,s: -z)
      doit(z,s,lambda z,s: z+s)
      doit(z,s,lambda z,s: s+z)
      doit(z,s,lambda z,s: s*z)
      doit(z,s,lambda z,s: z*s)
      doit(z,s,lambda z,s: z-s)
      doit(z,s,lambda z,s: s-z)
      doit(z,s,lambda z,s: z/s)
      doit(z,s,lambda z,s: s/z)
      doit(z,s,lambda z,s: z**s)
      doit(z,s,lambda z,s: s**z)
      doit(z,s,lambda z,s: np.fmin(s,z))
      doit(z,s,lambda z,s: np.fmax(s,z))
      doit(z,s,lambda z,s: constpow(s,z))
      doit(z,s,lambda z,s: constpow(z,s))

    nums = [np.array([[1,2],[3,4]]),DM([[1,2],[3,4]]), DM(4), np.array(4),4.0,4]

    ## numeric & SX
    for s in nums:
      for z in [SX.sym("x"), SX.sym("x"), SX.sym("x",2,2)]:
        print("z = %s, s = %s" % (str(z),str(s)))
        print("  z = %s, s = %s" % (type(z),type(s)))
        tests(z,s)

    # numeric & MX
    for s in nums:
      for z in [MX.sym("x",2,2)]:
        print("z = %s, s = %s" % (str(z),str(s)))
        print("  z = %s, s = %s" % (type(z),type(s)))
        tests(z,s)

    # SX & SX
    for s in [SX.sym("x"), SX.sym("x"), SX.sym("x",2,2)]:
      for z in [SX.sym("x"),SX.sym("x"), SX.sym("x",2,2)]:
        print("z = %s, s = %s" % (str(z),str(s)))
        print("  z = %s, s = %s" % (type(z),type(s)))
        tests(z,s)

    ## MX & MX
    for s in [MX.sym("x"),MX.sym("x",2,2)]:
      for z in [MX.sym("x"),MX.sym("x",2,2)]:
        print("z = %s, s = %s" % (str(z),str(s)))
        print("  z = %s, s = %s" % (type(z),type(s)))
        tests(z,s)

    for (s,x,y) in [
                  (np.array([[1,2],[3,4]]),SX.sym("x",2,2),MX.sym("x",2,2))
                  ]:
      for z,ztype in zip([x,y],[[type(SX()),type(SX())],[type(MX())]]):
        print("z = %s, s = %s" % (str(z),str(s)))
        print("  z = %s, s = %s" % (type(z),type(s)))
        doit(z,s,lambda z,s: -z)
        -s
        doit(z,s,lambda z,s: z+s)
        doit(z,s,lambda s,z: s+z)
        doit(z,s,lambda z,s: s*z)
        doit(z,s,lambda s,z: z*s)
        doit(z,s,lambda z,s: z-s)
        doit(z,s,lambda s,z: s-z)
        doit(z,s,lambda z,s: z/s)
        doit(z,s,lambda s,z: s/z)

if __name__ == '__main__':
    unittest.main()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs032():
    # The optimal objective is (if given in):
    x_opt = cs.DM([0, 0, 1])
    f_opt = 1
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    x0[0]= 0.1
    lbx[0]=0
    x0[1]= 0.7
    lbx[1]=0
    x0[2]= 0.2
    lbx[2]=0
    
    lbg[0] = 3
    g[0] = 6*x[1] + 4*x[2] - x[0]**3
    lbg[1] =  1
    ubg[1] =  1
    g[1] = x[0] + x[1] + x[2]
    obj = (x[0] + 3*x[1] + x[2])**2 + 4*(x[0] - x[1])**2
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *

# Consider the following simple Modelica file
with open('rocket.mo', 'r') as f: print(f.read())

# Different tools exist to compile Modelica models like these into FMUs.
# The following code shows how we could do it using the open-source tool 
# OpenModelica/OMPython, cf. the OpenModelica User's Guide.
from OMPython import OMCSessionZMQ
omc = OMCSessionZMQ()
if omc.loadFile('rocket.mo').startswith('false'):
  raise Exception('Modelica compilation failed: {}'.format(omc.sendExpression('getErrorString()')))
omc.sendExpression('setDebugFlags("-disableDirectionalDerivatives")')
fmu_file = omc.sendExpression('translateModelFMU(rocket)')
flag = omc.sendExpression('getErrorString()')
if not fmu_file.endswith('.fmu'): raise Exception('FMU generation failed: {}'.format(flag))
print("translateModelFMU warnings:\n{}".format(flag))

# Regardless of how the FMU was obtained, we must unzip it before CasADi can process it
# If there is an existing directory with the same name it is better to explicitly
# delete it to prevent potential memory corruption on overwriting.
import os, shutil, zipfile
unzipped_path = os.path.join(os.getcwd(), 'rocket_fmu_unzipped')
if os.path.isdir(unzipped_path): shutil.rmtree(unzipped_path)
with zipfile.ZipFile('rocket.fmu', 'r') as zip_ref: zip_ref.extractall(unzipped_path)

# Create a CasADi/DaeBuilder instance from the unzipped FMU
dae = DaeBuilder('rocket', unzipped_path)
dae.disp(True)

# Get state vector, initial conditions, bounds
x = dae.x()
lbx = dae.min(x)
ubx = dae.max(x)
x0 = dae.start(x)
print('x: ', x)
print('lbx: ', lbx)
print('ubx: ', ubx)
print('x0: ', x0)

# Get control vector, initial conditions, bounds
u = dae.u()
lbu = dae.min(u)
ubu = dae.max(u)
u0 = dae.start(u)
print('u: ', u)
print('lbu: ', lbu)
print('ubu: ', ubu)
print('u0: ', u0)

# Let's create a CasADi function for evaluating the ODE right-hand-side.
# We only need to specify the expressions that are varying:
f = dae.create('f', ['x', 'u'], ['ode'])

# This is a standard CasADi function that can be embedded into CasADi expressions
print(f)

# We can evaluate it and calculate derivatives as with other CasADi functions
ode0 = f(x0, u0)
print('ode0: ', ode0)

# Analytic first derivatives with sparsities are also available
jac = f.factory('jac_f', ['x', 'u'], ['jac:ode:x', 'jac:ode:u'])
jac_ode_x0, jac_ode_u0 = jac(x0, u0)
print('jac_ode_x0: ', jac_ode_x0)
print('jac_ode_u0: ', jac_ode_u0)

# Code generation for the functions are currently work in progress
f.generate('fmu_codegen')



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *

# Lotka-Volerra problem from mintoc.de

T = 12. # Time horizon
N = 40 # number of control intervals

# Model parameters
c0 = .4
c1 = .2

# Initial condition for x
x0 = [0.5, 0.7]

# Bounds on x
lbx = [0., 0.]
ubx = [2., 2.]

# Bounds on u
lbu = 0.
ubu = 1.

# Declare model variables
x = SX.sym('x', 2)
u = SX.sym('u')

# Model equations
xdot = vertcat(x[0] - x[0] * x[1] - c0 * x[0] * u,
              -x[1] + x[0] * x[1] - c1 * x[1] * u)

# Objective term
L = (x[0] - 1)**2 + (x[1] - 1)**2 + 1e-4*u**2

# Formulate discrete time dynamics
if False:
   # CVODES from the SUNDIALS suite
   dae = {'x':x, 'p':u, 'ode':xdot, 'quad':L}
   F = integrator('F', 'cvodes', dae, 0, T/N)
else:
   # Fixed step Runge-Kutta 4 integrator
   M = 4 # RK4 steps per interval
   DT = T/N/M
   f = Function('f', [x, u], [xdot, L])
   X0 = MX.sym('X0', 2)
   U = MX.sym('U')
   X = X0
   Q = 0
   for j in range(M):
       k1, k1_q = f(X, U)
       k2, k2_q = f(X + DT/2 * k1, U)
       k3, k3_q = f(X + DT/2 * k2, U)
       k4, k4_q = f(X + DT * k3, U)
       X=X+DT/6*(k1 +2*k2 +2*k3 +k4)
       Q = Q + DT/6*(k1_q + 2*k2_q + 2*k3_q + k4_q)
   F = Function('F', [X0, U], [X, Q],['x0','p'],['xf','qf'])

# Initial guess for u
u_start = [DM(0.)] * N

# Get a feasible trajectory as an initial guess
xk = DM(x0)
x_start = [xk]
for k in range(N):
    xk = F(x0=xk, p=u_start[k])['xf']
    x_start += [xk]

# Start with an empty NLP
w=[]
w0 = []
lbw = []
ubw = []
discrete = []
J = 0
g=[]
lbg = []
ubg = []

# "Lift" initial conditions
X0 = MX.sym('X0', 2)
w += [X0]
lbw += x0
ubw += x0
w0 += [x_start[0]]
discrete += [False, False]

# Formulate the NLP
Xk = X0
for k in range(N):
    # New NLP variable for the control
    Uk = MX.sym('U_' + str(k))
    w   += [Uk]
    lbw += [lbu]
    ubw += [ubu]
    w0  += [u_start[k]]
    discrete += [True]

    # Integrate till the end of the interval
    Fk = F(x0=Xk, p=Uk)
    Xk_end = Fk['xf']
    J=J+Fk['qf']

    # New NLP variable for state at end of interval
    Xk = MX.sym('X_' + str(k+1), 2)
    w   += [Xk]
    lbw += lbx
    ubw += ubx
    w0  += [x_start[k+1]]
    discrete += [False, False]

    # Add equality constraint
    g   += [Xk_end-Xk]
    lbg += [0, 0]
    ubg += [0, 0]

# Concatenate decision variables and constraint terms
w = vertcat(*w)
g = vertcat(*g)

# Create an NLP solver
nlp_prob = {'f': J, 'x': w, 'g': g}
nlp_solver = nlpsol('nlp_solver', 'bonmin', nlp_prob, {"discrete": discrete});
#nlp_solver = nlpsol('nlp_solver', 'knitro', nlp_prob, {"discrete": discrete});
#nlp_solver = nlpsol('nlp_solver', 'ipopt', nlp_prob); # Solve relaxed problem

# Plot the solution
tgrid = [T/N*k for k in range(N+1)]
import matplotlib.pyplot as plt
plt.figure(1)
plt.clf()
def plot_sol(w_opt):
    w_opt = w_opt.full().flatten()
    x0_opt = w_opt[0::3]
    x1_opt = w_opt[1::3]
    u_opt = w_opt[2::3]
    plt.plot(tgrid, x0_opt, '--')
    plt.plot(tgrid, x1_opt, '-')
    plt.step(tgrid, vertcat(DM.nan(1), u_opt), '-.')
    plt.xlabel('t')
    plt.legend(['x0','x1','u'])
    plt.grid(True)

# Solve the NLP
sol = nlp_solver(x0=vertcat(*w0), lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg)

print(nlp_solver.stats())

w1_opt = sol['x']
lam_w_opt = sol['lam_x']
lam_g_opt = sol['lam_g']
plot_sol(w1_opt)

plt.show()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs011():
    # The optimal objective is (if given in):
    f_opt = -8.498464223
    x = cs.MX.sym('x', 2)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(2, 1)
    lbx = -cs.inf*cs.DM.ones(2, 1)
    ubx = cs.inf*cs.DM.ones(2, 1)
    g = cs.MX.zeros(1, 1)
    lbg = -cs.inf*cs.DM.ones(1, 1)
    ubg = cs.inf*cs.DM.ones(1, 1)
    
    x0[0]= 4.9
    x0[1]= 0.1
    
    ubg[0] = 0
    g[0] = x[0]**2  - x[1]
    obj = (x[0] - 5)**2 + x[1]**2 -25

    #a = 7.5*cs.sqrt(6) + cs.sqrt(338.5)
    #x_opt = cs.DM([(a - 1/a)/cs.sqrt(6), (a**2 - 2 + a**(-2))/6])
    x_opt = [1.23477, 1.52466]
    
    f = obj
    print('hs011: Acc. to H&S optimal solution is [(a - 1/a)/cs.sqrt(6), (a**2 - 2 + a**(-2))/6]')
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
from numpy import random, array
import unittest
from types import *
from helpers import *
import itertools


class DocStrings(casadiTestCase):

  def test_1(self):
    doc = Function.jacobian.__doc__
    print(doc)
    self.assertTrue("Calculate all Jacobian blocks Generates a function that take" not in doc)

  def test_2(self):
    doc = cse.__doc__
    self.assertTrue("Common subexpression elimination" in doc)

  def test_3(self):
    doc = nlpsol.__doc__
    self.assertTrue("Create an NLP solver" in doc)
    self.assertTrue("WORHP" in doc)
    
  def test_4(self):
    doc = conic.__doc__
    self.assertTrue("Second-order cone constraints" in doc)
    self.assertTrue("olves QPs using a Mehrotra predictor-corrector interior poi" in doc)
    self.assertTrue("min_lam" in doc)
    self.assertTrue("dump_out" in doc)
    self.assertTrue("CONIC_A" in doc)
    
  def test_5(self):
    doc = collocation_interpolators.__doc__
    self.assertTrue("A collocation method" in doc)

  def test_6(self):
    doc = adj.__doc__
    self.assertTrue("Matrix adjoint" in doc)

  def test_7(self):
    doc = taylor.__doc__
    self.assertTrue("sin(x)" in doc)
    
  def test_8(self):
    doc = erfinv.__doc__
    self.assertTrue("Inverse error function" in doc)


if __name__ == '__main__':
    unittest.main()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#

import casadi as c
import numpy
from numpy import random, array, linalg, matrix, zeros, ones
import unittest
from types import *
from helpers import *
from casadi import *

scipy_available = True
try:
	from scipy.sparse import csc_matrix
except:
	scipy_available = False
import warnings

inherit_from = object

 
if has_nlpsol("ipopt"):
  inherit_from = casadiTestCase
  nlpsolver = "ipopt"
  nlpsolver_options = {}

class OptiStacktests(inherit_from):


    @requires_conic("qrqp")
    def test_conic(self):
      opti = Opti('conic')
      x = opti.variable(2)
      opti.minimize(dot(x,x))
      opti.subject_to(x[0]>=2)
      opti.subject_to(x[1]>=3)
      opti.solver('qrqp')

      sol = opti.solve()
      self.checkarray(sol.value(x),vertcat(2,3))

      with self.assertInException("conic"):
        Opti('foo')


    def test_lookup(self):
      opti = Opti()
      x = opti.variable(2)
      y = opti.variable(2)
      opti.subject_to(x>=y)

      str0 = str(opti.debug.x_lookup(0))
      str1 = str(opti.debug.x_lookup(1))
      str2 = str(opti.debug.x_lookup(2))
      str3 = str(opti.debug.x_lookup(3))
      
      self.assertEqual(str0,str1)
      self.assertEqual(str2,str3)
      
      self.assertFalse(str1==str2)
     
      
      
    def test_n(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y**2)**2)
      opti.subject_to(x+y>=1)
      opti.subject_to(z+x>=1)
      
      opti.solver('ipopt')
      sol = opti.solve()

      self.assertEqual(sol.opti.nx,3)
      self.assertEqual(sol.opti.ng,2)
      self.assertEqual(sol.opti.np,0)
      
    def test_subject_to_list(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y**2)**2)
      opti.subject_to([x+y>=1,z+x>=1])
      
      opti.solver('ipopt')
      sol = opti.solve()

      self.assertEqual(sol.opti.nx,3)
      self.assertEqual(sol.opti.ng,2)
      self.assertEqual(sol.opti.np,0)
      
    def test_sol_opti(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y**2)**2)
      opti.subject_to(x+y>=1)
      opti.subject_to(z+x>=1)
      
      opti.solver(nlpsolver,nlpsolver_options)

      sol = opti.solve()
      
      sol.opti.nx
      
    def test_to_function(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      p = opti.parameter()
      
      opti.minimize(y**2+sin(x-y-p)**2)
      opti.subject_to(x+y>=1)

      opti.solver(nlpsolver,nlpsolver_options)

      

      F = opti.to_function("F",[x,p,opti.lam_g],[x,y])
      
      r = F(0,0.1,0)
      
      print(F(0.1,0.1,0))
      print(F(0,2,0))
      print(F(100,1,0))

      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      p = opti.parameter()

      opti.minimize(sin(2*pi*(x+y-p))**2+sin(pi*z)**2)
      opti.subject_to(x==2*y)

      opti.solver("ipopt")

      F = opti.to_function("F",[vertcat(x,y,z),p],[vertcat(x,y,z)])

      self.checkarray(F([3.9,1.8,0],6),vertcat(4,2,0))
      self.checkarray(F([0,1,0],6),vertcat(2.0/3,1.0/3,0))
      self.checkarray(F([3.9,1.8,7.8],6),vertcat(4,2,8))
      self.checkarray(F([0,1,7.8],6),vertcat(2.0/3,1.0/3,8))

      F = opti.to_function("F",[vertcat(z,x,y),p],[vertcat(x,y,z)])
      self.checkarray(F([7.8,3.9,1.8],6),vertcat(4,2,8))
      self.checkarray(F([7.8,0,1],6),vertcat(2.0/3,1.0/3,8))


      b = opti.variable()
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      a = opti.variable()
      p = opti.parameter()

      opti.minimize(sin(2*pi*(x+y-p))**2+sin(pi*z)**2)
      opti.subject_to(x==2*y)

      opti.solver("ipopt")

      with self.assertInException("Argument 0"):
        opti.to_function("F",[vertcat(x,2)],[vertcat(x,y,z)])
      with self.assertInException("Argument 1"):
        opti.to_function("F",[y,vertcat(x,2)],[vertcat(x,y,z)])

      with self.assertInException("not independent"):
        opti.to_function("F",[x,x],[vertcat(x,y,z)])
      opti.to_function("F",[a],[vertcat(x,y,z)])
      with self.assertInException("belonging to a different instance"):
        opti.to_function("F",[b],[vertcat(x,y,z)])

    def test_dual(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y**2)**2)
      opti.subject_to(x+y>=1)
      opti.subject_to(z+x>=1)
      
      opti.solver(nlpsolver,nlpsolver_options)

      sol = opti.solve()
      
 

    def test_sparse(self):
      opti = Opti()
      x = opti.variable(3,1)
      p = opti.parameter()

      opti.minimize((p*x[1]-x[0]**2)**2)
      opti.subject_to(x[0]+3*x[1]==1)
      
      opti.solver(nlpsolver,nlpsolver_options)
      opti.set_value(p, 3)
      
      sol = opti.solve()
      
      if scipy_available:
        self.assertTrue(isinstance(sol.value(DM.eye(3)),csc_matrix))

    def test_structure(self):
      opti = Opti()
      x = opti.variable(3,1)
      p = opti.parameter()

      opti.minimize((p*x[1]-x[0]**2)**2)
      opti.subject_to(x[0]+3*x[1]==1)
      
      opti.solver(nlpsolver,nlpsolver_options)
      opti.set_value(p, 3)

      sol = opti.solve()
      
      jac_g = jacobian(opti.g,opti.x)
      self.checkarray(sol.value(jac_g),sol.value(sol.opti.debug.casadi_solver.get_function('nlp_jac_g')(opti.x,opti.p)[1]))
      grad_f = gradient(opti.f,opti.x)
      self.checkarray(sol.value(grad_f),sol.value(sol.opti.debug.casadi_solver.get_function('nlp_grad_f')(opti.x,opti.p)[1]))
      hess_lag = sol.value(hessian(opti.f+dot(opti.lam_g,opti.g),opti.x)[0])
      self.checkarray(sol.value(hess_lag),sol.value(tril2symm(sol.opti.debug.casadi_solver.get_function('nlp_hess_l')(opti.x,opti.p,1,opti.lam_g).T)))


    def test_warmstart(self):
      opti = Opti()

      x = opti.variable(2,1)
      p = opti.parameter()

      opti.minimize((p*x[1]-x[0]**2)**2)
      opti.subject_to(x[0]+3*x[1]==1)

      opti.solver(nlpsolver,nlpsolver_options)

      opti.set_value(p, 3)
      sol = opti.solve()
      
      sol1 = sol

      opti.set_value(p, 4)
      sol = opti.solve()
      
      with self.assertRaises(Exception):
        self.checkarray(sol.value(x),sol1.value(x))
      
      it = sol.stats()["iter_count"]
      
      self.assertTrue(it!=0)
      sol = opti.solve()
      self.assertTrue(sol.stats()["iter_count"]==it)
      
      opti.set_initial(sol.value_variables())
      sol = opti.solve()
      
      self.assertTrue(sol.stats()["iter_count"]!=it)
      opti.set_value(sol1.value_parameters())
      sol = opti.solve()
      self.checkarray(sol.value(x),sol1.value(x), digits=6)
      
      
    def test_set_value_expr(self):

      opti = Opti()
      p = opti.parameter(3,1)
      v = opti.parameter(2,1)
      x = opti.variable(2)
      
      opti.set_value(p, 0)
      opti.set_value(v, 0)
      opti.set_value(p[0], 3)
      self.checkarray(opti.debug.value(p),vertcat(3,0,0))
      opti.set_value(p[[0,2]], 2)
      self.checkarray(opti.debug.value(p),vertcat(2,0,2)) 
      opti.set_value(p[[0,2]], [1,2])
      self.checkarray(opti.debug.value(p),vertcat(1,0,2)) 
      opti.set_value(p[[2,0]], [1,2])
      self.checkarray(opti.debug.value(p),vertcat(2,0,1)) 

      opti.set_value(veccat(v,p), [1,2,3,4,5])
      self.checkarray(opti.debug.value(p),vertcat(3,4,5))
      self.checkarray(opti.debug.value(v),vertcat(1,2))    
     
     
      opti.set_value(p, 0)
      opti.set_value(veccat(p[0],p[0]), [4,4])
      self.checkarray(opti.debug.value(p),vertcat(4,0,0))
      with self.assertInException("ambiguous"):
        opti.set_value(veccat(p[0],p[0]), [4,5])
      with self.assertInException("cannot set a value for a variable"):
        opti.set_value(veccat(p,x,v), [4,5])
      opti.set_value(p, 0)
      opti.set_value(3*p[0], 3)
      self.checkarray(opti.debug.value(p),vertcat(1,0,0))
      with self.assertInException("cannot set initial/value of an arbitrary expression"):
        opti.set_value(p[0]+p[1], 3)

      opti.set_value(veccat(v,3), [1,2,3])
      opti.set_value(veccat(2,v,5,p), [2,3,4,5,6,7,8])
      opti.set_value(veccat(2,v,5,p,v), [2,3,4,5,6,7,8,3,4])
      
      with self.assertInException("inconsistent"):
        opti.set_value(veccat(v,3), [1,2,4])
      with self.assertInException("inconsistent"):
        opti.set_value(veccat(2,v,5,p), [5,3,4,2,6,7,8])
      with self.assertInException("ambiguous"):
        opti.set_value(veccat(2,v,5,p,v), [2,3,4,5,6,7,8,4,3])
          
    def test_shapes(self):
        
          opti = Opti()
          
          F = opti.variable(10,1)
          x = opti.variable(2,11)
         
          opti.subject_to(x[:,0]==x[:,1]) 
          opti.subject_to(opti.bounded(-1,F,1))

          opti.solver(nlpsolver,nlpsolver_options)
          opti.minimize(sum1(F))
          sol = opti.solve()

    def test_symb_boundedshapes(self):
        
          opti = Opti()
          
          y = opti.variable()
          x = opti.variable()
         
          opti.subject_to(opti.bounded(0,y,x))

          opti.solver(nlpsolver,nlpsolver_options)
          opti.minimize(x**2+(y-6)**2)
          
          opti.subject_to(x<=3)
          sol = opti.solve()
          
          self.checkarray(sol.value(y), 3, digits=3)

    def test_symb_parametric(self):
        
          opti = Opti()
          
          y = opti.parameter(3,1)
          x = opti.parameter(3,1)
         
          with self.assertInException("!parametric[0]"):
            opti.subject_to(x<=y)

          
    def test_callback(self):
        
          opti = Opti()
          
          eps = 1e-5 
          
          x = opti.variable()
          y = opti.variable()
          
          p = opti.parameter()
          
          opti.minimize((x-1)**4+(y-p)**4)
          opti.solver(nlpsolver,nlpsolver_options)

          opti.callback(lambda i: ret.setdefault('a',opti.debug.value(p)))
          opti.set_value(p, 3)
          ret = {}
          sol = opti.solve()

          self.assertTrue(ret['a']==3)
          opti.set_value(p, 2)
          ret = {}
          sol = opti.solve()
          self.assertTrue(ret['a']==2)
          opti.set_value(p, 3)
          ret = {}
          sol = opti.solve()
          self.assertTrue(ret['a']==3)
          
          opti.callback()
          ret = {}
          sol = opti.solve()
          self.assertFalse('a' in ret)

          opti.callback(lambda i: ret.setdefault('b',opti.debug.value(p)))
          sol = opti.solve()
          self.assertTrue(ret['b']==3)
          
          
          opti = opti.copy()
          ret = {}
          sol = opti.solve()
          self.assertTrue(len(ret)==0)

    def test_debug_value(self):
      
        opti = Opti()
        p = opti.parameter()
        opti.set_value(p, 3)
        self.checkarray(opti.debug.value(p**2), 9)
        x = opti.variable()
        
        with self.assertInException("This action is forbidden since you have not solved the Opti stack yet"):
          opti.debug.value(x**2)
        
        with self.assertInException("You cannot set a value for a variable"):
          opti.set_value(x, 2)
        with self.assertInException("You cannot set a value for a variable"):
          opti.set_value([x==2])
        opti.set_initial(x, 2)
        with self.assertInException("You cannot set an initial value for a parameter"):
          opti.set_initial(p, 2)
        with self.assertInException("You cannot set an initial value for a parameter"):
          opti.set_initial([p==2])
        
        
        with self.assertInException("This action is forbidden since you have not solved the Opti stack yet"):
          opti.debug.value(x**2)
        
        self.checkarray(opti.debug.value(x**2, opti.initial()), 4)
        
        y = opti.variable()
        
        with self.assertInException("This action is forbidden since you have not solved the Opti stack yet"):
          opti.debug.value(x*y)
      
    def test_print(self):
      opti = Opti()
      print(opti)
      self.assertTrue("variables" in str(opti))
      self.assertTrue("variables" in str(opti.debug))
      x = opti.variable()
      y = opti.variable()
      p = opti.parameter()
      print(opti)
      
      opti.minimize((x**2-y)**2)
      print(opti)
      opti.solver(nlpsolver,nlpsolver_options)
      print(opti)
      sol = opti.solve()
      print(sol)
      print(opti)      
            
      opti = Opti()
      print(opti)
      x = opti.variable()
      y = opti.variable()
      
      p = opti.parameter()
      print(opti)
      
      opti.subject_to((x**2-y)**2<=1)
      print(opti)
      opti.solver(nlpsolver,nlpsolver_options)
      print(opti)
      sol = opti.solve()
      print(sol)
      print(opti)
      
      self.assertTrue("variables" in str(sol))
            
    def test_flow(self):
      opti = Opti()
      
      x = opti.variable()
      y = opti.variable()
      
      p = opti.parameter()
      
      w = MX.sym("w")
      
      opti.minimize(x**2)
      opti.solver(nlpsolver,nlpsolver_options)
      sol = opti.solve()
      
      with self.assertInException("do not appear in the constraints and objective"):
        sol.value(y)
      with self.assertInException("decision variable"):
        sol.value(y)
      with self.assertInException("optistack.py"):
        sol.value(y) 
      with self.assertInException("MX symbol 'w' of shape 1x1, declared outside of Opti."):
        sol.value(w)
        
      with self.assertInException("MX symbol 'w' of shape 1x1, declared outside of Opti."):
        opti.subject_to(w<=2)

      with self.assertInException("MX symbol 'w' of shape 1x1, declared outside of Opti."):
        opti.minimize(w)
        
      opti.minimize((x-p)**2)

      with self.assertInException("You have forgotten to assign a value to a parameter"):
        sol = opti.solve()
      with self.assertInException("Opti parameter"):
        sol = opti.solve()
      with self.assertInException("optistack.py"):
        sol = opti.solve()
      opti.set_value(p, 5)
      opti.solve()
      opti.debug.value(x);
      
      opti.set_value(p, 9)
      opti.debug.value(p);
      with self.assertInException("This action is forbidden since you have not solved the Opti stack yet"):
          opti.debug.value(x);
      
      opti.solve()
      opti.set_initial(x, 3)
      opti.debug.value(p);
      with self.assertInException("This action is forbidden since you have not solved the Opti stack yet"):
          opti.debug.value(x);     
      
      con = x>=0
      opti.subject_to(con)
      opti.solve()
            
      opti0 = opti

      opti = Opti()
      
      X = opti.variable()
      y = opti.variable()
      
      P = opti.parameter()
      with self.assertInException("belonging to a different instance of Opti"):
        opti.subject_to(x<=2)

      with self.assertInException("decision variable"):
        opti.subject_to(x<=2)

      with self.assertInException("belonging to a different instance of Opti"):
        opti.subject_to(p<=2)

      with self.assertInException("parameter"):
        opti.subject_to(p<=2)
     
      with self.assertInException("belonging to a different instance of Opti"):
        opti.subject_to(opti0.dual(con)<=2)

      with self.assertInException("dual variable"):
        opti.subject_to(opti0.dual(con)<=2)
        
      con = X>=0
      opti.subject_to(con)
      dual = opti.dual(con)
      
      with self.assertInException("Dual variables forbidden in this context"):
        opti.subject_to(dual>=2)
      with self.assertInException("Dual variables forbidden in this context"):
        opti.minimize(dual)
        
      opti.subject_to()
      opti.minimize(y**2)
      dual = opti.dual(con)
      opti.solver(nlpsolver,nlpsolver_options)
      sol = opti.solve()
      
      with self.assertInException("This expression has a dual for a constraint that is not given to Opti"):
        sol.value(dual)
      with self.assertInException("optistack.py"):
        sol.value(dual)
      
        
        
    def test_simple(self):
      
        opti = Opti()
        
        eps = 1e-5 
        
        x = opti.variable()
        y = opti.variable()
      
        f = (x-1)**2+(y-2)**2  
        opti.minimize(f)

        opti.solver(nlpsolver,nlpsolver_options)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1)
        self.checkarray(sol.value(y), 2)
      

        count = 0
        for con, coneps, xs, ys, mul in [
          (y>=2.5, y+eps>=2.5, 1, 2.5, 1),
          (y>=1.5, y+eps>=1.5, 1, 2, 0),
          (2.5>=y, 2.5+eps>=y, 1, 2, 0),
          (1.5>=y, 1.5+eps>=y, 1, 1.5, 1),
          (y<=2.5, y<=2.5+eps, 1, 2, 0),
          (y<=1.5, y<=1.5+eps, 1, 1.5, 1),
          (2.5<=y, 2.5<=y+eps, 1, 2.5, 1),
          (1.5<=y, 1.5<=y+eps, 1, 2, 0),
          (y>=x,   y+eps>=x, 1, 2, 0),
          (y<=x,   y<=x+eps, 1.5, 1.5, 1),
          (y<=0,   y<=eps, 1, 0, 4),
          ((3<= y) <=4, (3<= y+eps) <=4, 1, 3, None),
          (3<= (y <=4), 3<= (y+eps <=4), 1, 3, None),
          (0<= (y <=1), 0<= (y <=1+eps), 1, 1, None),
          ((0<= y) <=1, (0<= y) <=1+eps, 1, 1, None),
          ((4>= y) >=3, (4>= y+eps) >=3, 1, 3, None),
          ((1>= y) >=0, (1+eps>= y) >=0, 1, 1, None),
          (opti.bounded(3,y,4), opti.bounded(3,y+eps,4), 1, 3, None),
          (opti.bounded(0,y,1), opti.bounded(0,y+eps,1), 1, 1, None)
        
          
          
          ]:
          opti.subject_to()
          opti.subject_to(con)
          sol = opti.solve()
  

          self.checkarray(sol.value(x), xs,digits=7)
          self.checkarray(sol.value(y), ys,digits=7)
        
          dual_value = sol.value(opti.dual(con))
          opti.subject_to()
          opti.subject_to(coneps)
          sol2 = opti.solve()

          s = -sign(sol2.value(f)-sol.value(f))
          
          count+=1
          if mul is not None:
            self.checkarray(dual_value, s*abs(mul),digits=6)
        
        opti.subject_to()
        opti.subject_to(1.5==y)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)
        
        opti.subject_to()
        opti.subject_to(y==1.5)
        sol = opti.solve()
        
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)

        opti.subject_to()
        opti.subject_to(y==x)
        sol = opti.solve()
        
        x = opti.variable(3,3)
        f = trace(x)
        opti.subject_to()
        opti.subject_to(x>=0)
        with self.assertRaises(Exception):
          sol = opti.solve()
        
        x = opti.variable(2,1)
        opti.subject_to()
        opti.minimize((x[0]-1)**2+(x[1]-2)**2)
        opti.subject_to(x[1]>=x[0])
        sol = opti.solve()
        self.checkarray(sol.value(x), vertcat(1,2),digits=7)
          
        opti.subject_to()
        opti.subject_to(x[1]<=x[0])
        sol = opti.solve()
        
        self.checkarray(sol.value(x), vertcat(1.5,1.5),digits=7)
        
        opti.subject_to()
        opti.subject_to(x<=0.5)
        sol = opti.solve()
        
        print("test", sol.value(x))
        self.checkarray(sol.value(x), vertcat(0.5,0.5),digits=7)

    def test_parametric(self):
      
        opti = Opti()
        
        x = opti.variable()
        y = opti.variable()
        p = opti.parameter()
        
        opti.minimize((x-1)**2+(y-p)**2)
        opti.solver(nlpsolver,nlpsolver_options)
        opti.set_value(p, 2)
        sol = opti.solve()

        self.checkarray(sol.value(x), 1)
        self.checkarray(sol.value(y), 2)

        opti.set_value(p, 3)
        sol = opti.solve()


        self.checkarray(sol.value(x), 1)
        self.checkarray(sol.value(y), 3)
        
        opti.minimize((x-1)**2+(y-2)**2)
        
        opti.subject_to()
        opti.subject_to(y>=p)
        opti.set_value(p, 2.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2.5,digits=7)
        
        opti.set_value(p, 1.5)
        sol = opti.solve()
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2,digits=7)
        
        opti.subject_to()
        opti.subject_to(p>=y)
        opti.set_value(p, 2.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2,digits=7)
        
        opti.set_value(p, 1.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)
        
        
        opti.subject_to()
        opti.subject_to(y<=p)
        opti.set_value(p, 2.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2,digits=7)
        
        opti.set_value(p, 1.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)
        
        opti.subject_to()
        opti.subject_to(p<=y)
        opti.set_value(p, 2.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2.5,digits=7)
        
        opti.set_value(p, 1.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 2,digits=7)
        
        
        opti.subject_to()
        opti.subject_to(p==y)
        opti.set_value(p, 1.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)
        
        opti.subject_to()
        opti.subject_to(y==p)
        opti.set_value(p, 1.5)
        sol = opti.solve()
        
        self.checkarray(sol.value(x), 1,digits=7)
        self.checkarray(sol.value(y), 1.5,digits=7)
        
    def test_symm(self):
      
        opti = Opti()
        
        P = opti.variable(2, 2, 'symmetric')
        
        R = DM([[1,2],[4,4]])
        
        f = sum2(sum1((P-R)**2))

        x = symvar(f)[0]

        opti.minimize(f)
        opti.solver(nlpsolver,nlpsolver_options)

        sol = opti.solve()
        
        F = casadi.Function('f',[x],[f])

        self.checkarray(sol.value(P), DM([[1,3],[3,4]]))

        P = opti.variable(2, 2)
        
        
        f = sum2(sum1((P-R)**2))

        x = symvar(f)[0]

        
        opti.minimize(f)
        sol = opti.solve()
        
        F = casadi.Function('f',[x],[f])

        self.checkarray(sol.value(P), R)

    def test_broadcast(self):
        opti = Opti()
        
        x = opti.variable(3, 1)
        

        f = mtimes(x.T,x)
        
        opti.minimize(f)
        
        opti.subject_to(x[0]>=vertcat(1,2,3))
        opti.solver(nlpsolver,nlpsolver_options)
        sol = opti.solve()
        self.checkarray(sol.value(x[0]), 3,digits=7)
        

        f = mtimes(x.T,x)
        
        opti.minimize(f)
        
        opti.subject_to();
        
        opti.subject_to(x[0]>=vertcat(1+x[1],0.5,0))
        opti.solver(nlpsolver,nlpsolver_options)
        sol = opti.solve()
        self.checkarray(sol.value(x[0]), 0.5,digits=4)

    def test_constraint_dim_mismatch(self):
        opti = Opti()
        x = opti.variable(5,1)

        p = opti.parameter(1,1)
        q = opti.parameter(1,1)


        opti.minimize(dot(x,x))
        with self.assertInException("Constraint must contain decision variables."):
          opti.subject_to(p==q)
        with self.assertInException("Constraint shape mismatch."):
          opti.subject_to(x[0]==vertcat(1,2,3))
        with self.assertInException("Constraint shape mismatch."):
          opti.subject_to(vertcat(1,2,3)==x[0])


    def test_value(self):
        opti = Opti()
        x = opti.variable()
        y = opti.variable()
        z = opti.variable()
        p = opti.parameter()
        q = opti.parameter()
        g = x-p>=0
        opti.subject_to(g)
        opti.minimize(x**2+(y-3)**2)
        lam = opti.dual(g)
        
        self.assertEqual(opti.debug.value(p**3,[p==2]),8)
        self.assertEqual(opti.debug.value(x**3,[x==2]),8)
        self.assertEqual(opti.debug.value(x+p,[x==2,p==3]),5)
        self.assertEqual(opti.debug.value(lam**3,[opti.lam_g==2]),8)
        with self.assertInException("This expression depends on a parameter with unset value"):
          opti.debug.value(p**3)
        with self.assertInException("This action is forbidden since you have not solved"):
          opti.debug.value(x**3)
        with self.assertInException("This action is forbidden since you have not solved"):
          opti.debug.value(lam**3)
        opti.solver("ipopt")
        
        opti.set_value(p,2)
        sol = opti.solve()
        self.assertEqual(opti.debug.value(p**2),4)
        self.assertEqual(opti.debug.value(p**2,[p==3]),9)
        self.checkarray(opti.debug.value(x**2),4,digits=5)
        self.checkarray(opti.debug.value(x**2,[x==3]),9)
        self.checkarray(opti.debug.value(lam**2),16,digits=5)
        self.checkarray(opti.debug.value(lam**2,[opti.lam_g==1]),1,digits=5)
        with self.assertInException("This expression has symbols that do not appear in the constraints and objective:"):
          opti.debug.value(z**3)
        self.assertEqual(opti.debug.value(z**3,[z==2]),8)
        with self.assertInException("This expression depends on a parameter with unset value"):
          opti.debug.value(q)

    def test_introspection(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      p = opti.parameter()
      
      opti.minimize((x-y**2)**2)
      opti.subject_to(x+y+p>=1)
      opti.subject_to(z+x>=1)
      
      opti.solver('ipopt')
      opti.set_value(p, 3)
      sol = opti.solve()

      solver = nlpsol("solver","ipopt",{"x": opti.x, "f": opti.f, "g": opti.g, "p": opti.p})
      sol2 = solver(p=sol.value(p),lbg=sol.value(opti.lbg),ubg=sol.value(opti.ubg))
      
      self.checkarray(sol2["x"],sol.value(opti.x))
      
    def test_max_iter(self):
      opti = Opti()
      x = opti.variable()
      opti.minimize((x-1)**6)
      opti.solver('ipopt',{},{"max_iter":1})
      with self.assertInException("Maximum_Iterations_Exceeded"):
        sol = opti.solve()

      sol = opti.solve_limited()
      opti.solver('ipopt',{},{"max_iter":1000})
      opti.subject_to(x>=1)
      opti.subject_to(x<=-1)

      with self.assertInException("Infeasible"):
        sol = opti.solve()

      with self.assertInException("Infeasible"):
        sol = opti.solve_limited()

    @requires_conic("superscs")
    def test_conic(self):
      options = {"eps":1e-9,"do_super_scs":1, "verbose":0}

      opti = Opti('conic')
      x = opti.variable()
      y = opti.variable()

      #  min  2 x + y
      #
      #    ||  x-5 , y-7 ||_2 <= 4
      #
      #
      for x_scale in [1,7]:
        for g_scale in [1,9]:
          opti.set_linear_scale(x,x_scale)

          h = soc(vertcat(x-5,y-7),4)

          # Note: >= destroys sparsity
          opti.subject_to(h>0,g_scale)
      
          opti.minimize(2*x+y)

          opti.solver("superscs",{},options)
          sol = opti.solve()

          res = sol.value(vertcat(x,y))

          self.checkarray(res,DM([5-8/sqrt(5),7-4/sqrt(5)]),conic,digits=7)
          self.checkarray(sol.value(opti.f),10-16/sqrt(5)+7-4/sqrt(5),conic,digits=7)
   
    @requires_conic("cbc")
    def test_discrete_linear(self):
      opti = Opti('conic')
      x = opti.variable()
      y = opti.variable()
      
      opti.minimize(x+y)
      opti.subject_to(y>=0.5*x+1)
      opti.subject_to(y<=2*x+3)
      
      #opti.subject_to(-5<= (x <= 5))
      #opti.subject_to(-5<= (y <= 5))
      #opti.subject_to(-5<= (y <= 5))
      
      opti.set_domain(x, "real")
      opti.solver("cbc")
      
      sol = opti.solve()
      
      f_real = sol.value(opti.f)
      xsol_real = sol.value(x)
      print(f_real,xsol_real)
      opti.set_domain(x, "integer")
      
      sol = opti.solve()
      
      f_integer = sol.value(opti.f)
      xsol_integer = sol.value(x)
      print(f_integer,xsol_integer)
      self.checkarray(round(xsol_integer), xsol_integer)
      self.assertTrue(f_integer> f_real)
 
    @requires_nlpsol("bonmin")
    def test_discrete(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      p = opti.parameter()
      opti.set_value(p,-1)
      
      opti.minimize(x**2+y**2)
      opti.subject_to(x+y+p>=0)
      
      opti.set_domain(x, "real")
      opti.solver("bonmin")
      
      sol = opti.solve()
      
      f_real = sol.value(opti.f)
      xsol_real = sol.value(x)
      opti.set_domain(x, "integer")
      
      sol = opti.solve()
      
      f_integer = sol.value(opti.f)
      xsol_integer = sol.value(x)
      self.checkarray(round(xsol_integer), xsol_integer)
      self.assertTrue(f_integer> f_real)
      
      f = opti.to_function('f',[p],[x])
      self.checkarray(f(-1), xsol_integer)
      
    @requires_nlpsol("ipopt")
    def test_linear_scale(self):
    
      x = MX.sym("x")
      

      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y)**2)
      opti.subject_to(x+y>=1)
      opti.subject_to(z+x>=1)
      
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)
      
      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_grad_f": {"final_options":{"dump_out":True}},"nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_unscaled = sol.value(opti.x)
      
      x_unscaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_unscaled = DM.from_file("nlp_f.000000.out.f.mtx")
      grad_f_unscaled = DM.from_file("nlp_grad_f.000000.out.grad_f_x.mtx")
      jac_g_unscaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y)**2)
      opti.subject_to(x+y>=1)
      opti.subject_to(z+x>=1)
      
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)
      
      opti.set_linear_scale(x,10)
      opti.set_linear_scale(y,5,1)
      opti.set_linear_scale(z,3,1)
      
      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_grad_f": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_scaled = sol.value(opti.x)
      
      x_scaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_scaled = DM.from_file("nlp_f.000000.out.f.mtx")
      grad_f_scaled = DM.from_file("nlp_grad_f.000000.out.grad_f_x.mtx")
      
      self.checkarray((x_unscaled-vertcat(0,1,1))/vertcat(10,5,3),x_scaled)
      self.checkarray(f_unscaled, f_scaled)
      self.checkarray(x_sol_scaled, x_sol_unscaled)
      self.checkarray(grad_f_unscaled*vertcat(10,5,3),grad_f_scaled)

      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y)**2)
      opti.subject_to(x+y>=1, 10)
      opti.subject_to(z+x>=1, 3)
      
      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()

      x_scaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_scaled = DM.from_file("nlp_f.000000.out.f.mtx")
      jac_g_scaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      self.checkarray(x_sol_scaled, x_sol_unscaled)
      
      self.checkarray(jac_g_scaled[0,:]*10, jac_g_unscaled[0,:])
      self.checkarray(jac_g_scaled[1,:]*3, jac_g_unscaled[1,:])

    @requires_nlpsol("ipopt")
    def test_linear_scale2(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y-z)**2)
      opti.subject_to(z-3 <= vertcat(x,y))
      opti.subject_to(vertcat(x,y) <= z+3)
      
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)
      
      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_grad_f": {"final_options":{"dump_out":True}},"nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_unscaled = sol.value(opti.x)
      
      x_unscaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_unscaled = DM.from_file("nlp_f.000000.out.f.mtx")
      grad_f_unscaled = DM.from_file("nlp_grad_f.000000.out.grad_f_x.mtx")
      jac_g_unscaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      

      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      
      opti.minimize((x-y-z)**2)
      opti.subject_to(z-3 <= vertcat(x,y))
      opti.subject_to(vertcat(x,y) <= z+3)
    
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)

      opti.set_linear_scale(x,10)
      opti.set_linear_scale(y,5,1)
      opti.set_linear_scale(z,3,1)

      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_scaled = sol.value(opti.x)

      x_scaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_scaled = DM.from_file("nlp_f.000000.out.f.mtx")
      jac_g_scaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      
      self.checkarray(x_sol_scaled, x_sol_unscaled)
      self.checkarray(jac_g_scaled[:,0], jac_g_unscaled[:,0]*10)
      self.checkarray(jac_g_scaled[:,1], jac_g_unscaled[:,1]*5)
      self.checkarray(jac_g_scaled[:,2], jac_g_unscaled[:,2]*3)
  
    @requires_nlpsol("ipopt")
    def test_linear_scale3(self):
      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      opti.minimize((x-y-z)**2)
      opti.subject_to(z-3 <= (vertcat(x,y) <= z+3))
      
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)
      
      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_grad_f": {"final_options":{"dump_out":True}},"nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_unscaled = sol.value(opti.x)
      
      x_unscaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_unscaled = DM.from_file("nlp_f.000000.out.f.mtx")
      grad_f_unscaled = DM.from_file("nlp_grad_f.000000.out.grad_f_x.mtx")
      jac_g_unscaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      

      opti = Opti()
      x = opti.variable()
      y = opti.variable()
      z = opti.variable()
      
      
      opti.minimize((x-y-z)**2)
      opti.subject_to(z-3 <= (vertcat(x,y) <= z+3))
    
      opti.set_initial(x,1)
      opti.set_initial(y,2)
      opti.set_initial(z,1)

      opti.set_linear_scale(x,10)
      opti.set_linear_scale(y,5,1)
      opti.set_linear_scale(z,3,1)

      opti.solver("ipopt",{"specific_options": {"nlp_f": {"final_options":{"dump_in":True,"dump_out":True}}, "nlp_jac_g": {"final_options":{"dump_out":True}}},"ipopt.nlp_scaling_method":"none","ipopt.mumps_permuting_scaling":0,"ipopt.mumps_scaling":0})

      sol = opti.solve()
      
      x_sol_scaled = sol.value(opti.x)

      x_scaled = DM.from_file("nlp_f.000000.in.x.mtx")
      f_scaled = DM.from_file("nlp_f.000000.out.f.mtx")
      jac_g_scaled = DM.from_file("nlp_jac_g.000000.out.jac_g_x.mtx")
      
      self.checkarray(x_sol_scaled, x_sol_unscaled)
      self.checkarray(jac_g_scaled[:,0], jac_g_unscaled[:,0]*10)
      self.checkarray(jac_g_scaled[:,1], jac_g_unscaled[:,1]*5)
      self.checkarray(jac_g_scaled[:,2], jac_g_unscaled[:,2]*3)
      
    def test_detect_simple_bounds(self):
  
      checks = ["f","g","lbg","ubg","lam_g"]
      
      ref = {}
      for scale_x in [None,1,9]:
        for scale_g in [None,1, 100]:
          for detect_simple_bounds in [False,True]:
            opti = Opti()
            x = opti.variable()
            y = opti.variable()
            z = opti.variable()
            
            if scale_x is not None:
              opti.set_linear_scale(x,scale_x)

            opti.set_initial(x,12)
            opti.set_initial(y,12)
            opti.set_initial(z,12)

            if scale_g is None:
              opti.subject_to(-5 <= (x <= 7))
            else:
              opti.subject_to(-5 <= (x <= 7), scale_g)

            opti.subject_to(-3 <= (-y <= 2))
            opti.subject_to(-2 <= (-y <= 1))
            opti.subject_to(-3 <= (-0.3*z <= 2))
            opti.subject_to(-1 <= (-0.3*z <= 0))
            opti.subject_to(-4 <= (x <= 6))

            opti.minimize((x-10)**2+(y-10)**2+(z-10)**2)

            opti.solver("ipopt",{"detect_simple_bounds": detect_simple_bounds})

            sol = opti.solve()
            
            for k in checks:
              if scale_x is None and scale_g is None and not detect_simple_bounds:
                ref[k] = sol.value(getattr(opti,k))
              else:
                self.checkarray(ref[k], sol.value(getattr(opti,k)), digits=6)

    @memory_heavy()
    @requires_nlpsol("ipopt")
    def test_ipopt_custom_jac(self):
    
      # f_scale
      for x_scale in [vertcat(1,1,1),vertcat(7,0.11,0.13)]:
        for x_scale_offset in [vertcat(0,0,0),vertcat(1,2,3)]:
          for g_scale in [vertcat(1,1),vertcat(3,5)]:
            for f_scale in [1,1.7]:
              for scale_helper in [True,False]:
                opti = Opti()
                x=opti.variable()
                y=opti.variable()
                w=opti.variable()

                
                opti.set_linear_scale(x,x_scale[0],x_scale_offset[0])
                opti.set_linear_scale(y,x_scale[1],x_scale_offset[1])
                opti.set_linear_scale(w,x_scale[2],x_scale_offset[2])
                
                opti.minimize((1-x)**2+100*w**2,f_scale)
                opti.subject_to(y-x**2-w==0,g_scale[0])
                opti.subject_to(w**2-y==7,g_scale[1])
                
                opti.solver("ipopt")
                opti.solve()
               

                ref_solver = opti.debug.casadi_solver


                opti = Opti()
                x=opti.variable()
                y=opti.variable()
                w=opti.variable()
                
                
                opti.set_linear_scale(x,x_scale[0],x_scale_offset[0])
                opti.set_linear_scale(y,x_scale[1],x_scale_offset[1])
                opti.set_linear_scale(w,x_scale[2],x_scale_offset[2])
                
                opti.minimize((1-x)**2+100*w**2,f_scale)
                opti.subject_to(y-x**2-w==0,g_scale[0])
                opti.subject_to(w**2-y==7,g_scale[1])
                

                
                if scale_helper:
                  nlp_jac_g_custom = opti.scale_helper(Function('nlp_jac_g',[opti.x,opti.p],[opti.g,jacobian(opti.g,opti.x)],["x","p"],["g","jac_g_x"]))
                else:
                  nlp_jac_g_custom = Function('nlp_jac_g',[opti.x,opti.p],substitute([opti.g/opti.g_linear_scale,mtimes(jacobian(opti.g/opti.g_linear_scale,opti.x),diag(opti.x_linear_scale))],[opti.x],[opti.x*opti.x_linear_scale+opti.x_linear_scale_offset]),["x","p"],["g","jac_g_x"])

                options = {}
                options["cache"] = {"nlp_jac_g":nlp_jac_g_custom}
                opti.solver("ipopt", options)
                try:
                  sol = opti.solve()
                except:
                  pass
                
                solver = opti.debug.casadi_solver
                
                #print(sol.value(opti.f+dot(opti.lam_g, opti.g)))

                print(opti.x_linear_scale)
                print(opti.x_linear_scale_offset)
                print(opti.g_linear_scale)
                print(opti.f_linear_scale)
                self.checkfunction_light(ref_solver.get_function("nlp_jac_g"),solver.get_function("nlp_jac_g"),inputs=[vertcat(0.11,0.3,0.7),0])
                lam_f = MX.sym("lam_f")
                if scale_helper:
                  lag = lam_f*opti.f+dot(opti.lam_g,opti.g)
                  H = jacobian(gradient(lag,opti.x),opti.x,{"symmetric":True})
                  nlp_hess_l_custom = opti.scale_helper(Function('nlp_hess_l',[opti.x,opti.p,lam_f,opti.lam_g],[triu(H)],["x","p","lam_f","lam_g"],["triu_hess_gamma_x_x"]))
                else:
                  lag = lam_f*opti.f/f_scale+dot(opti.lam_g,opti.g/opti.g_linear_scale)
                  H = mtimes(jacobian(gradient(lag,opti.x),opti.x,{"symmetric":True}),diag(opti.x_linear_scale)**2)
                  H = substitute(triu(H),opti.x,opti.x*opti.x_linear_scale+opti.x_linear_scale_offset)
                  nlp_hess_l_custom = Function('nlp_hess_l',[opti.x,opti.p,lam_f,opti.lam_g],[H])
                  
                print(ref_solver.get_function("nlp_hess_l"))
                self.checkfunction_light(ref_solver.get_function("nlp_hess_l"),nlp_hess_l_custom,inputs=[vertcat(0.11,0.3,0.7),0,3,17])

    @memory_heavy()
    @requires_nlpsol("ipopt")
    def test_scaling(self):
    
      f_ref = None
      g_ref = None
      x_ref = None
      lam_g_ref = None
      grad_f_ref = None
      jac_g_ref = None
      
      dual_ref = None
      lbg_ref = None
      for x_scale in [1,100]:
        for g_scale in [1,37]:
          for f_scale in [1,11]:
            print(x_scale,g_scale)

            opti = Opti()
            x=opti.variable()
            y=opti.variable()
            w=opti.variable()
            
            
            opti.set_linear_scale(x,x_scale)
            
            
            opti.minimize((1-x)**2+100*(y-x**2)**2,f_scale)
            my_g = x**2+y**2==1
            opti.subject_to(my_g,g_scale)
            
            opti.solver("ipopt",{"ipopt.print_level":0,"print_time":False})
            sol = opti.solve()
            
            print(sol.value(opti.lbg))
            
            if x_scale==1 and g_scale==1 and f_scale==1:
              f_ref = sol.value(opti.f)
              g_ref = sol.value(opti.g)
              x_ref = sol.value(opti.x)
              lam_g_ref = sol.value(opti.lam_g)
              grad_f_ref = sol.value(gradient(opti.f,opti.x))
              jac_g_ref = sol.value(jacobian(opti.g,opti.x))
              dual_ref = sol.value(opti.dual(my_g))
              lbg_ref = sol.value(opti.lbg)
            else:
              self.checkarray(f_ref,sol.value(opti.f))
              self.checkarray(g_ref,sol.value(opti.g))
              self.checkarray(x_ref,sol.value(opti.x))
              self.checkarray(lam_g_ref,sol.value(opti.lam_g),digits=6)
              self.checkarray(grad_f_ref, sol.value(gradient(opti.f,opti.x)),digits=6)
              self.checkarray(jac_g_ref, sol.value(jacobian(opti.g,opti.x)),digits=6)
              self.checkarray(dual_ref, sol.value(opti.dual(my_g)),digits=6)
              self.checkarray(lbg_ref, sol.value(opti.lbg))
            
            lag_grad = sol.value(gradient(opti.f+dot(opti.lam_g, opti.g),opti.x))
            self.checkarray(sol.value(x), 0.78641515, digits=6)
            self.checkarray(lag_grad, vertcat(0,0), digits=6)

    @requires_conic("highs")
    @requires_nlpsol("ipopt")
    def test_dual(self):
    
        ref = {}
        
        for t in ['nlp','conic']:
            opti = Opti(t)
            x = opti.variable()
            y = opti.variable()
            z = opti.variable()
            opti.minimize(x**2+y**2+z**2)
            g1 = 2*x+3*y+4<=0
            opti.subject_to(g1)
            g2 = 7*x+3*z+6==0
            opti.subject_to(g2)
            opti.solver('highs' if t=='conic' else 'ipopt')
            
            sol = opti.solve()
            
            if t=='conic':
                data = {}
            else:
                data = ref
            
            data["x"] = sol.value(opti.x)
            data["lam_g"] = sol.value(opti.lam_g)
            data["g1"] = sol.value(opti.dual(g1))
            data["g2"] = sol.value(opti.dual(g2))
        print(data)
        print(ref)
        for k in data.keys():
            self.checkarray(data[k],ref[k],digits=6)
    
    @requires_conic("superscs")
    @requires_nlpsol("ipopt")
    def test_dual_soc(self):
    
        ref = {}
        
        for t in ['conic']:
            opti = Opti(t)
            x = opti.variable()
            y = opti.variable()
            z = opti.variable()
            h = soc(vertcat(x-5,y-7),4)

            # Note: >= destroys sparsity
            opti.subject_to(h>0)
            g1 = 1*x+2*y+3*z+3==0
            opti.subject_to(g1)
        
            opti.minimize(2*x+y)
            
            if t=='conic':
                opti.solver('superscs',{"superscs": {"eps":1e-9,"do_super_scs":1, "verbose":0}})
            else:
                opti.solver("ipopt")

            sol = opti.solve()
            
            if t=='conic':
                data = {}
            else:
                data = ref
            data["f"] = sol.value(opti.f)
            data["x"] = sol.value(opti.x)
            data["lam_g"] = sol.value(opti.lam_g)
            #data["g1"] = sol.value(opti.dual(g1))
            #data["g2"] = sol.value(opti.dual(g2))
            print(data)


     # to_function with lam_g
if __name__ == '__main__':
    unittest.main()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
from casadi import *

# Example on how to use the DaeBuilder class
# Joel Andersson, UW Madison 2017

# Start with an empty DaeBuilder instance
dae = DaeBuilder('rocket')

# Add input expressions
a = dae.add('a', 'parameter', 'tunable')
b = dae.add('b', 'parameter', 'tunable')
u = dae.add('u', 'input')
h = dae.add('h')
v = dae.add('v')
m = dae.add('m')

# Constants
g = 9.81 # gravity

# Set ODE right-hand-side
dae.eq(dae.der(h), v)
dae.eq(dae.der(v), (u-a*v**2)/m-g)
dae.eq(dae.der(m), -b*u**2)

# Specify initial conditions
dae.set_start('h', 0)
dae.set_start('v', 0)
dae.set_start('m', 1)

# Add meta information
dae.set_unit('h','m')
dae.set_unit('v','m/s')
dae.set_unit('m','kg')

# Print DAE
dae.disp(True)



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs029():
    # The optimal objective is (if given in):
    x_opt = cs.DM([4, 2*cs.sqrt(2), 2])
    f_opt = -16*cs.sqrt(2)
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(1, 1)
    lbg = -cs.inf*cs.DM.ones(1, 1)
    ubg = cs.inf*cs.DM.ones(1, 1)
    
    x0[0]= 1
    x0[1]= 1
    x0[2]= 1
    
    ubg[0] = 48
    g[0] = x[0]**2 + 2*x[1]**2 + 4*x[2]**2
    obj = -x[0]*x[1]*x[2]
    
    
    f = obj
    print('hs029: Possible optimal points could be (2, -2sqrt(2), -2), (-2, 2sqrt(2), -2), (-4, -2sqrt(2), 2)')
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from doxy2swig import *

import sys

import ipdb
import texttable
import re

import lxml.etree as ET
import pydot
import os
cwd = os.getcwd()

import pathlib
stem = str(pathlib.Path(cwd).parent.parent.parent)

expression_tools = set()


for r in open("../../../swig/casadi.i","r"):
  if "casadi_" in r:
    m = re.search("casadi_(\w+)", r)
    if m:
      expression_tools.add(m.group(1))

aliases = {}
for line in open('../Doxyfile.in','r'):
  if line.startswith('ALIASES'):
    m = re.search('\+=\s*(\w+)\s*=\s*"(.*?)"',line) 
    if m:
      aliases[m.group(1)]=m.group(2)

print(aliases)


def astext(node,whitespace=False,escape=True,strictspacing=False):
  r = []
  if node.nodeType == node.TEXT_NODE:
    d = node.data
    if escape:
      d = d.replace('\\', r'\\\\')
      d = d.replace('"', r'\"')
    if not(whitespace):
      d = d.strip()
    r.append(d)
  elif hasattr(node,'tagName') and node.tagName=="sp":
    return " "
  elif hasattr(node,'childNodes'):
    for node in node.childNodes:
      r.append(astext(node,whitespace=whitespace,escape=escape,strictspacing=strictspacing))
  if strictspacing:
    return ("".join(r))
  else:
    return (" ".join(r)).strip()


class Doxy2SWIG_X(Doxy2SWIG):

  def __init__(self, *args, **kwargs):
    self.internal = kwargs["internal"]
    del kwargs["internal"]
    self.deprecated = kwargs["deprecated"]
    del kwargs["deprecated"]
    self.merge = kwargs["merge"]
    del kwargs["merge"]
    self.groupdoc = kwargs["groupdoc"]
    del kwargs["groupdoc"]
    Doxy2SWIG.__init__(self,*args, **kwargs)
    

    # skip sharedobjectnode stuff
    for e in self.xmldoc.getElementsByTagName("compounddef"):
      for e2 in e.getElementsByTagName("inheritancegraph"):
        for e3 in e2.getElementsByTagName("node"):
          for e4 in e3.getElementsByTagName("label"):
            if "Node" in e4.firstChild.data:
              print("skipped")
              try:
                self.xmldoc.removeChild(e)
              except:
                pass

    self.docstringmap = {}
    self.active_docstring = None
    self.add_text_counter = 0
    self.src = args[0]
    del self.ignores[self.ignores.index("programlisting")]


  def generic_parse(self, node, pad=0):
        """A Generic parser for arbitrary tags in a node.

        Parameters:

         - node:  A node in the DOM.
         - pad: `int` (default: 0)

           If 0 the node data is not padded with newlines.  If 1 it
           appends a newline after parsing the childNodes.  If 2 it
           pads before and after the nodes are processed.  Defaults to
           0.

        """
        npiece = 0
        if pad:
            npiece = self.add_text_counter
            if pad == 2:
                self.add_text('\n')                
        for n in node.childNodes:
            self.parse(n)
        if pad:
            if self.add_text_counter > npiece:
                self.add_text('\n')
                
  def clean_pieces(self, pieces):
      """Cleans the list of strings given as `pieces`.  It replaces
      multiple newlines by a maximum of 2 and returns a new list.
      It also wraps the paragraphs nicely.
      
      """
      ret = []
      count = 0
      for i in pieces:
          if i == '\n':
              count = count + 1
          else:
              if i == '";':
                  if count:
                      ret.append('\n')
              elif count > 2:
                  ret.append('\n\n')
              elif count:
                  ret.append('\n'*count)
              count = 0
              ret.append(i)

      _data = "".join(ret)
      ret = []
      for i in _data.split('\n\n'):
          if i == 'Parameters:' or i == 'Exceptions:':
              ret.extend([i, '\n-----------', '\n\n'])
          elif i.find('// File:') > -1: # leave comments alone.
              ret.extend([i, '\n'])
          else:
              if i.strip().startswith(">") or "---" in i or "===" in i or "%%newline%%" in i:
                if "%%newline%%" in i:
                  _tmp = i.replace("%%newline%%","\n")
                else:
                  _tmp = i.strip()
              else:
                _tmp = textwrap.fill(i.strip(), 80-4, break_long_words=False,drop_whitespace=False,replace_whitespace=False)
              _tmp = self.lead_spc.sub(r'\1"\2', _tmp)
              ret.extend([_tmp, '\n\n'])
      return ret
        
  def write(self, fname):
        o = my_open_write(fname)
        if self.multi:
            for p in self.pieces:
              pp = p#.encode("ascii","ignore")
              pp = re.sub("[A-Z_]*_EXPORT ","",pp)
              o.write(pp)
        else:
            for p in self.clean_pieces(self.pieces):
              pp = p#.encode("ascii","ignore")
              pp = re.sub("[A-Z_]*_EXPORT ","",pp)
              o.write(pp)
        o.close()
        
  def do_doxygenindex(self, node):
      self.multi = 1
      comps = node.getElementsByTagName('compound')
      for c in comps:
          refid = c.attributes['refid'].value
          if c.attributes['kind'].value in ["example","struct","source","dir","file"]: continue
          filters = ["internal","interface","node"]
          filtered = False          
          for f in filters:
            if f in refid or f.capitalize() in refid: filtered = True
          if "SparsityInterface" in refid: filtered = False
          if filtered: continue

          print(c.attributes['refid'].value, c.attributes['kind'].value)

          fname = refid + '.xml'
          if not os.path.exists(fname):
              fname = os.path.join(self.my_dir,  fname)
          if fname.endswith("cpp.xml"): continue
          if not self.quiet:
              print("parsing file: %s"%fname)
          p = Doxy2SWIG_X(fname, self.include_function_definition, self.quiet,internal=self.internal,deprecated=self.deprecated,merge=self.merge,groupdoc=self.groupdoc)
          p.generate()
          self.pieces.extend(self.clean_pieces(p.pieces))
  
  def do_hruler(self,node):
    self.add_text("\n\n" + "-"*80 + "\n\n")

  def do_heading(self,node):
    level = int(node.attributes['level'].value)-2
    text = astext(node)
    length = len(text)
    self.add_text(text+"\n")
    if level==1:
      self.add_text(("="*length) + "\n\n")
    else:
      self.add_text(("-"*length) + "\n\n")

  def do_htmlonly(self,node):
    pass
    
  def do_programlisting(self,node):
    if True or hasattr(node.previousSibling,'tagName') and node.previousSibling.tagName=="htmlonly" and "doctest" in node.previousSibling.firstChild.data:
      self.add_text("\n\n::\n\n")
      for codeline in node.getElementsByTagName("codeline"):
        self.add_text("  >>> " + astext(codeline,strictspacing=True).replace("\n","%%newline%%")+"\n")
    
    
  def do_verbatim(self, node):
    skipheader = False
    try:
      if node.previousSibling.tagName=="htmlonly" and "doctest" in node.previousSibling.firstChild.data:
        skipheader = True
    except:
      pass
    if not skipheader:
      self.add_text("\n\n::\n\n")
    text = node.firstChild.data
    text = "\n".join(["  " +i for i in text.split("\n")])
    
    self.add_text(text.replace("\n","%%newline%%").replace('\\', r'\\\\').replace('"', r'\"')+"\n\n")
    
  def do_table(self, node):
    caption = node.getElementsByTagName("caption")
    if len(caption)==1:
      self.add_text(">" + astext(caption[0]).encode("ascii","replace").decode("ascii")+"\n\n")
    
    rows = []
    for (i,row) in enumerate(node.getElementsByTagName("row")):
      rows.append([])
      for (j,entry) in enumerate(row.getElementsByTagName("entry")):
        rows[i].append(astext(entry,escape=False).encode("ascii","replace").decode("ascii").replace("&gt;",">").replace("&lt;","<"))
      
    table = texttable.Texttable(max_width=80-4)
    table.add_rows(rows)
    
    d = table.draw()
    d = d.replace('\\', r'\\\\')
    d = d.replace('"', r'\"')
    self.add_text(d)
    self.add_text("\n")
    #print table.draw()
        
    #for row in rows:
    #  self.add_text("*")
    #  r = " "
    #  for col in row:
    #    r+=col+" | "
    #  self.add_text(col[:-1]+"\n")
    #self.generic_parse(node, pad=1)

  def do_compoundname(self, node):
      self.add_text('\n\n')
      data = node.firstChild.data
      self.start_docstring(data)

  def do_compounddef(self, node):
      kind = node.attributes['kind'].value

      if kind in ('class', 'struct'):
          prot = node.attributes['prot'].value
          if prot != 'public':
              return
          names = ('compoundname', 'briefdescription',
                   'detaileddescription', 'includes')
          first = self.get_specific_nodes(node, names)
          for n in names:
              if n in first:
                  self.parse(first[n])
          self.end_docstring()
          for n in node.childNodes:
              if n not in first.values():
                  self.parse(n)
      elif kind in ('file', 'namespace'):
          nodes = node.getElementsByTagName('sectiondef')
          for n in nodes:
              self.parse(n)
      elif kind in ('group',):
          if node.getElementsByTagName('compoundname')[0].firstChild.data.startswith("plugin_"):
            names = ('compoundname', 'briefdescription',
                     'detaileddescription', 'includes')
            first = self.get_specific_nodes(node, names)
            for n in names:
                if n in first:
                    self.parse(first[n])
            self.end_docstring()

  def do_sectiondef(self, node):
      kind = node.attributes['kind'].value
      if kind in ('public-func', 'func', 'user-defined', '','friend'):
          self.generic_parse(node)   

  def do_memberdef(self, node):
      prot = node.attributes['prot'].value
      id = node.attributes['id'].value
      kind = node.attributes['kind'].value
      tmp = node.parentNode.parentNode.parentNode
      compdef = tmp.getElementsByTagName('compounddef')[0]
      cdef_kind = compdef.attributes['kind'].value
      location = node.getElementsByTagName('location')[0]
      if "declfile" in location.attributes:
        location = location.attributes['declfile'].value
      else:
        location = location.attributes['file'].value
      if "bodyfile" in node.getElementsByTagName('location')[0].attributes:
          bodylocation = node.getElementsByTagName('location')[0].attributes['bodyfile'].value
          bodystart = int(node.getElementsByTagName('location')[0].attributes['bodystart'].value)
          bodyend = int(node.getElementsByTagName('location')[0].attributes['bodyend'].value)
      line = int(node.getElementsByTagName('location')[0].attributes['line'].value)
      if prot == 'public' and not location.endswith('cpp'):
          first = self.get_specific_nodes(node, ('definition', 'name','argsstring'))

          name = first['name'].firstChild.data
          if name[:8] == 'operator': # Don't handle operators yet.
              return

          if not ('definition' in first) or \
                 kind in ['variable', 'typedef']:
              return


         
            
          anc = node.parentNode.parentNode
          if cdef_kind in ('file', 'namespace'):
              ns_node = anc.getElementsByTagName('innernamespace')
              if not ns_node and cdef_kind == 'namespace':
                  ns_node = anc.getElementsByTagName('compoundname')
              if ns_node:
                  ns = ns_node[0].firstChild.data
                  target = '%s::%s'%(ns, name)
              else:
                  target = '%s '%(name)
          elif cdef_kind in ('class', 'struct'):
              # Get the full function name.
              anc_node = anc.getElementsByTagName('compoundname')
              cname = anc_node[0].firstChild.data
              target = '%s::%s'%(cname, name)

          definition = first['definition'].firstChild.data
          #definition of friends is missing parent class
          if not re.search("::~?\w+(<[ \w]+>)?$",definition):
            print("repair", definition)
            definition = " ".join(definition.split(" ")[:-1]+[target])
          try:
            defn = definition + first['argsstring'].firstChild.data
          except:
            return
            
          meta = {"decl": {"file":location, "line": line}}
          if "bodyfile" in node.getElementsByTagName('location')[0].attributes:
            meta["impl"] = {"file": bodylocation, "lines": (bodystart,bodyend)}
              
          self.start_docstring(target,defn,meta=meta)
          for n in node.childNodes:
              if n not in first.values():
                  self.parse(n)
          self.end_docstring()

  def start_docstring(self,target,origin="huma kavula",meta=None):
    self.active_docstring = (target,origin,meta)
    if target in self.docstringmap:
      self.docstringmap[target].append((origin,[],meta))
    else:
      self.docstringmap[target]= [(origin,[],meta)]
    
  def end_docstring(self):
    self.active_docstring = None
  
  def add_text(self,value):
    self.add_text_counter += 1
    if self.active_docstring is None:
      self.add_text_original(value)
    else:
      #print self.active_docstring, self.docstringmap[self.active_docstring[0]], value
      if type(value) in (list, tuple):
          self.docstringmap[self.active_docstring[0]][-1][1].extend(value)
      else:
          self.docstringmap[self.active_docstring[0]][-1][1].append(value)
    
  def add_text_original(self, value):
      Doxy2SWIG.add_text(self,value)
      
  def generate(self):
      try:
        publicapi = ET.parse(self.src.replace("_internal","_cluttered"))
      except Exception as e:
        publicapi = None
      Doxy2SWIG.generate(self)
      for k, v in self.docstringmap.items():
        if k.startswith("plugin_"):
          self.groupdoc[k] = v[0][1]
          break
        # Group together
        grouped_list = []
        grouped_dict = {}
        
        def fix_signature(a): return re.sub(" *?= *?(delete|default) *?$","", a.replace("override",""))
        
        for (origin,pieces,meta) in v:
          origin_nostatic = origin.replace("static ","")
          m = list(re.finditer("\[DEPRECATED(:(.*?))?\]","".join(pieces)))
          deprecated = ("" if m[-1].group(2) is None else m[-1].group(2)) if len(m)>0 else None
          
          if origin == "huma kavula" and publicapi is not None:
            internal = False
          else:
            internal = True
            if publicapi is not None:
              if publicapi.xpath(".//memberdef[translate(concat(definition,argsstring),' ','')='%s' or translate(concat(definition,argsstring),' ','')='%s']" % (origin_nostatic.replace(" ",""),'static'+origin_nostatic.replace(" ",""))):
                internal = False
          if deprecated is not None: internal = False
          
          if not re.search(r"\b_\w+\(",origin_nostatic) and "(" in origin_nostatic and "~" not in origin_nostatic:
            if origin_nostatic.endswith("=0"):
              swigname = origin_nostatic[:-2]
            else:
              swigname = origin_nostatic
            # Remove return type
            
            nest=0
            sep = 0
            for i,c in enumerate(swigname):
              if c=="<" : nest+=1
              if c==">" : nest-=1
              if c == " " and nest==0: sep = i
              if c=="(" : break
            
            swigname= swigname[sep:]
          else:
            swigname = ""
            
          if meta is not None:
              if len("".join(pieces).strip())>0:
                  pieces = pieces+["Doc source: https://github.com/casadi/casadi/blob/main%s#L%d\n" % (meta["decl"]["file"].replace(stem,""),meta["decl"]["line"])]
                  if "impl" in meta:
                      pieces = pieces+["Implementation: https://github.com/casadi/casadi/blob/main%s#L%d-L%d\n" % (meta["impl"]["file"].replace(stem,""),meta["impl"]["lines"][0],meta["impl"]["lines"][1])]
            
          #"INTERNAL": "mark_internal(\"$decl\");"
          #"DEPRECATED": "deprecated(\"%s\");"
          #"UNSAFE": "unsafe(\"%s\");"
                            
          if internal:
            pieces = ["[INTERNAL] "] + pieces
            if "*" not in swigname and swigname not in self.internal:
            # self.internal.add("%rename(\"_internal_%s\") " + swigname + ";")
            #  print swigname
              #self.internal.add("%pythonprepend " + swigname + " %{\n _mark_internal() \n%}")
              self.internal[swigname] = "%exception " + swigname + " {\n CATCH_OR_NOT(INTERNAL_MSG() $action) \n}"
          else:
            self.internal[swigname] = ""
            
          if deprecated is not None:
            self.deprecated[swigname] = "%exception " + swigname + " {\n CATCH_OR_NOT(DEPRECATED_MSG(\"%s\") $action)\n}" % deprecated
          else:
            self.deprecated[swigname] = ""
              
          total = u"".join(pieces)
          totalnowrap = total.replace("\n"," ")
          if (aliases["noswig"] in totalnowrap) or (aliases["nopython"] in totalnowrap):
             print("skipping", origin)
             continue
          if total in grouped_dict:
             grouped_dict[total][0].append(origin)
          else:
             grouped_dict[total] = ([origin],pieces)
             grouped_list.append(grouped_dict[total])
          if not self.merge:
            target = fix_signature(swigname if len(swigname)>0 else k)
            self.doc_target(target, pieces)
        if self.merge:
          target = fix_signature(k)
          
          if len(grouped_list)==1:
            self.doc_target(target, grouped_list[0][1])
          else:
            pieces = []
            for (origin,pieces) in grouped_list:
              if len(u"".join(pieces).rstrip())>0:
                pieces+=["\n"]+["\n>  " + o.replace('"',r'\"') + '\n'  for o in origin] + ["-"*(80-8) + "\n"] + pieces + ["\n"]
            pieces+=["\";\n","\n"]
            self.doc_target(target, pieces)


  def doc_target(self,target,content,correction=True):
    if len("".join(content).strip()) > 0: 
      self.add_text_original(["%feature(\"docstring\") ", target, " \"\n\n"]+content+["\";\n","\n"])
    if not correction: return
  def doc_target(self,target,content,correction=True):
    labels = ["SparsityInterface","Matrix","GenericMatrix","GenericExpression"]
    for label in labels:
      target = re.sub(r"\b" + label + " *<[\w ]+>",label+"Common", target)
    for label in labels:
      target = re.sub(r"\b" + label + r"\b",label+"Common", target)
     

    if len("".join(content).strip()) > 0: 
      self.add_text_original(["%feature(\"docstring\") ", target, " \"\n\n"]+content+["\";\n","\n"])
    if not correction or "Internal" in target or "::" not in target: return
    m = re.search(r"\b(\w+)\(",target)
    if m:
      if m.group(1) in expression_tools:
        content = [c.replace("[INTERNAL]","") for c in content]
        if "Functions called by friend functions defined" in " ".join(content):
            content = []
        content = [re.sub("Functions called by friend functions defined (here|for +\w+)\.?","",c) for c in content]
        self.doc_target("casadi::casadi_" + m.group(1), content,correction=False)
        target = target.split("(")[0]
        target = "::".join(target.split("::")[:-1])
        self.doc_target(target + "::casadi_" + m.group(1), content,correction=False)

def convert(input, output,  include_function_definition=True, quiet=False,internal=None,deprecated=None,merge=False,groupdoc=None):
    p = Doxy2SWIG_X(input, include_function_definition, quiet,internal=internal,deprecated=deprecated,merge=merge,groupdoc=groupdoc)
    p.generate()
    p.write(output)

    
if __name__ == '__main__':
    usage = __doc__
    parser = optparse.OptionParser(usage)
    parser.add_option("-n", '--no-function-definition',
                      action='store_true',
                      default=False,
                      dest='func_def',
                      help='do not include doxygen function definitions')
    parser.add_option("-q", '--quiet',
                      action='store_true',
                      default=False,
                      dest='quiet',
                      help='be quiet and minimize output')
                      
    parser.add_option("-m", '--merge',
                      action='store_true',
                      default=False,
                      dest='merge',
                      help='merge overloaded names')
    
    options, args = parser.parse_args()
    if len(args) != 4:
        parser.error("error: no input and output specified")
    
    internal = dict()
    deprecated = dict()
    groupdoc = dict()
    convert(args[0], args[1], False, options.quiet,internal=internal,deprecated=deprecated,merge=options.merge,groupdoc=groupdoc)
    open(args[2],'w').write("\n".join(sorted(filter(lambda x: len(x)>0, internal.values()))))
    open(args[3],'w').write("\n".join(sorted(filter(lambda x: len(x)>0, deprecated.values()))))
    import pickle
    filemap = pickle.load(open('filemap.pkl','rb'))
    
    for k,v in groupdoc.items():
      fn,n = filemap[k]
      f = open(fn.replace(".hpp","_meta.cpp"),"w")
      f.write(open('../../../misc/license_header.txt','r').read())
      f.write("""
      #include "%s"
      #include <string>

      const std::string %s::meta_doc=
      """ % (fn.split("/")[-1],n,))
      
      for p in "".join(v).split("\n"):
        f.write('"')
        f.write(p.replace('%%newline%%','\\n"\n"')+"\\n")
        f.write('"\n')
      f.write(';\n')
      f.close()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

# nlpsol
# =====================

from casadi import *
from numpy import *

# In this example, we will solve a few optimization problems with increasing complexity
#
# Scalar unconstrained problem
# ============================
# $\text{min}_x \quad \quad (x-1)^2$ \\
# subject to $-10 \le x \le 10$ \\
#
# with x scalar

x=SX.sym('x')
nlp = {'x':x, 'f':(x-1)**2}

solver = nlpsol('solver', 'ipopt', nlp)
sol = solver(lbx=-10, ubx=10)

# The solution is obviously 1:

print(sol['x'])
assert(abs(sol['x']-1)<1e-9)

# Constrained problem
# ============================
# $\text{min}_x \quad \quad (x-1)^T.(x-1)$ \\
# subject to $-10 \le x \le 10$ \\
# subject to $0 \le x_1 + x_2 \le 1$ \\
# subject to $ x_0 = 2$ \\
#
# with $x \in \mathbf{R}^n$

n = 5

x=SX.sym('x',n)

# Note how we do not distinguish between equalities and inequalities here

nlp = {'x':x, 'f':mtimes((x-1).T,x-1), 'g':vertcat(x[1]+x[2],x[0])}

solver = nlpsol('solver', 'ipopt', nlp)
sol = solver(lbx=-10, ubx=10, lbg=[0,2], ubg=[1,2])

# $ 2 \le x_0 \le 2$ is not really as bad it looks. 
# Ipopt will recognise this situation as an equality constraint.

# The solution is obviously [2,0.5,0.5,1,1]:

print(sol['x'])
for (i,e) in zip(list(range(n)),[2,0.5,0.5,1,1]):
  assert(abs(sol['x'][i]-e)<1e-7)


# Problem with parameters
# ============================
# $\text{min}_x \quad \quad (x-a)^2$ \\
# subject to $-10 \le x \le 10$ \\
#
# with x scalar

x=SX.sym('x')
a=SX.sym('a')
a_ = 2
nlp={'x':x, 'p':a, 'f':(x-a)**2}

solver = nlpsol('solver', 'ipopt', nlp)
sol = solver(lbx=-10, ubx=10, p=a_)

# The solution is obviously a:

print(sol['x'])
assert(abs(sol['x']-a_)<1e-9)

# The parameter can change inbetween two solve calls:

sol = solver(lbx=-10, ubx=10, p=2*a_)

# The solution is obviously 2*a:

print(sol['x'])
assert(abs(sol['x']-2*a_)<1e-9)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *

# Test problem
#
#    min x^2 + y^2
#    s.t.    x + y - 10 = 0
#

# Optimization variables
x = MX.sym("x")
y = MX.sym("y")

# Objective
f = x*x + y*y

# Constraints
g = x+y-10

# Create an NLP problem structure
nlp = {"x": vertcat(x,y), "f": f, "g": g}

mode = "jit"

# Pick a compiler
compiler = "gcc"    # Linux
# compiler = "clang"  # OSX
# compiler = "cl.exe" # Windows

# Run this script in an environment that recognised the compiler as command.
# On Windows, the suggested way is to run this script from a "x64 Native Tools Command Promt for VS" (Requires Visual C++ components or Build Tools for Visual Studio, available from Visual Studio installer. You also need SDK libraries in order to access stdio and math.)

flags = ["-O3"] # Linux/OSX

for mode in ["jit","external"]:

  if mode=="jit":
    # By default, the compiler will be gcc or cl.exe
    jit_options = {"flags": flags, "verbose": True, "compiler": compiler}
    options = {"jit": True, "compiler": "shell", "jit_options": jit_options}
    
    # Create an NLP solver instance
    solver = nlpsol("solver", "ipopt", nlp, options)

  elif mode=="external":
    # Create an NLP solver instance
    solver = nlpsol("solver", "ipopt", nlp)

    # Generate C code for the NLP functions
    solver.generate_dependencies("nlp.c")

    import subprocess
    # On Windows, use other flags
    cmd_args = [compiler,"-fPIC","-shared"]+flags+["nlp.c","-o","nlp.so"]
    subprocess.run(cmd_args)

    # Create a new NLP solver instance from the compiled code
    solver = nlpsol("solver", "ipopt", "./nlp.so")

  arg = {}

  arg["lbx"] = -DM.inf()
  arg["ubx"] =  DM.inf()
  arg["lbg"] =  0
  arg["ubg"] =  0
  arg["x0"] = 0

  # Solve the NLP
  res = solver(**arg)

  # Print solution
  print("-----")
  print("objective at solution =", res["f"])
  print("primal solution =", res["x"])
  print("dual solution (x) =", res["lam_x"])
  print("dual solution (g) =", res["lam_g"])
